/*

Directory handling

*/


#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include "ssdisk.h"
#include "dir.h"
#include "sscmd.h"
#include "diskio.h"


unsigned char *entrytype[] = {
  "del", // DIRTYPE_DEL   0
  "seq", // DIRTYPE_SEQ   1
  "prg", // DIRTYPE_PRG   2
  "usr", // DIRTYPE_USR   3
  "rel", // DIRTYPE_REL   4
  "cbm", // DIRTYPE_CBM   5
  "dir", // DIRTYPE_DIR   6
  "???"  // DIRTYPE_OTHER 7
};


/*

Read directory

*/
directory *getdir(unsigned char direction, unsigned char *buffer, unsigned char device) {
  unsigned int length;

  if (direction == SSTOCBM) {
    length = getssdir(buffer);
  } else {
    length = getiecdir(buffer, device);
  }

  if (length == 0) {
    return(NULL);
  }

  return(parsedir(buffer, length));
}


/*

Free memory allocated to a dir structure

*/
void freedir(directory *dir) {
  direntry *entry;
  direntry *nextentry;

  entry = dir->firstentry;
  free(dir);

  while (entry) {
    nextentry = entry->next;
    free(entry->name);
    free(entry);
    entry = nextentry;
  }
}


/*

Parse directory

*/
directory *parsedir(unsigned char *buffer, unsigned int length) {
  unsigned char *q1;
  unsigned char *q2;
  directory *dir;
  direntry *newentry;
  direntry *entry = NULL;
  unsigned int bufptr = 2;
  unsigned char i = 0;
  unsigned int linenum = 0;
  unsigned char namebuf[128];
  unsigned char name[MAXNAMELEN+1];
  unsigned char type[4];

  if ((dir = malloc(sizeof(directory))) == NULL) {
    return(NULL);
  }

  dir->numentries = 0;
  dir->name = "";
  dir->firstentry = NULL;
  dir->current = 0;
  dir->offset = 0;

  if (buffer[0] != 0x01 || buffer[1] != 0x04) {
    free(dir);
    return(NULL);
  }

  while (bufptr < length) {
    if (buffer[bufptr] == 0 && buffer[bufptr+1] == 0) {
      bufptr = length;
    } else {
      bufptr += 2;
      linenum = buffer[bufptr] + buffer[bufptr+1] * 256;
      bufptr += 2;
      if (buffer[bufptr] != 18) {
	i = 0;
	while (namebuf[i++] = buffer[bufptr++]) { ; }

	name[0] = 0;
	i = 0;
	if (q1 = strchr(namebuf, '"')) {
	  if ((q2 = strrchr(namebuf, '"')) != q1) {

	    /* q1++??? */
	    for (++q1; q1 < q2 && i < MAXNAMELEN; ++q1) {
	      name[i++] = q1[0];
	    }
	    name[i] = 0;

	    ++q2;
	    while (q2[0] == ' ') {
	      ++q2;
	    }
	    if (*q2 == '>' || *q2 == '*') {
	      ++q2;
	    }
	    type[0] = q2[0];
	    type[1] = q2[1];
	    type[2] = q2[2];
	    type[3] = 0;

	    if ((newentry = malloc(sizeof(direntry))) == NULL) {
	      return(dir);
	    }

	    newentry->prev = entry;
	    newentry->next = NULL;
	    newentry->size = linenum;

	    newentry->name = malloc(strlen(name) + 1);
	    strcpy(newentry->name, name);

	    newentry->type = DIRTYPE_OTHER;

	    if (strcmp(type, "prg") == 0) {
	      newentry->type = DIRTYPE_PRG;
	    } else if (strcmp(type, "dir") == 0) {
	      newentry->type = DIRTYPE_DIR;
	    } else if (strcmp(type, "seq") == 0) {
	      newentry->type = DIRTYPE_SEQ;
	    } else if (strcmp(type, "usr") == 0) {
	      newentry->type = DIRTYPE_USR;
	    } else if (strcmp(type, "rel") == 0) {
	      newentry->type = DIRTYPE_REL;
	    } else if (strcmp(type, "cbm") == 0) {
	      newentry->type = DIRTYPE_CBM;
	    } else if (strcmp(type, "del") == 0) {
	      newentry->type = DIRTYPE_DEL;
	    }

	    if (entry) {
	      entry->next = newentry;
	    }
	    entry = newentry;
	    ++(dir->numentries);
	  }
	}
      }
    }
  }

  if (entry) {
    while (entry->prev) {
      entry = entry->prev;
    }
    dir->firstentry = entry;
  }

  return(dir);
}


/*

Get a pointer to the entrynumth direntry

*/
direntry *getentry(directory *dir, unsigned char entrynum) {
  direntry *de;

  de = dir->firstentry;
  while(entrynum) {
    if (de->next) {
      de = de->next;
      --entrynum;
    } else {
      return(NULL);
    }
  }
  return(de);
}
