/*

Disk I/O

*/


#include <string.h>
#include <stdlib.h>
#include <cbm.h>
#include <conio.h>
#include <stdio.h>
#include <errno.h>

#include "ssdisk.h"
#include "geometry.h"
#include "diskio.h"


/*

Write a track to the drive

*/
unsigned char writetrack(unsigned char device, unsigned char *buffer, unsigned char track, unsigned char blockcount, unsigned char drivetype) {
  unsigned char s, c, x, col;
  int l;
  unsigned char cmd[16];
  unsigned char written[MAXBLOCKS];

  x = wherex();

  for (s = 0; s < MAXBLOCKS; ++s) {
    written[s] = 0;
  }

  c = blockcount;

  if ((s = cbm_open(15, device, 15, "")) != 0) {
    printf("\nopen15,%d,15 failed: %d\n", device, s);
    return(ERROR);
  }
  if ((s = cbm_open(2, device, 2, "#")) != 0) {
    printf("\nopen2,%d,2,\"#\" failed: %d\n", device, s);
    cbm_close(15);
    return(ERROR);
  }

  s = 0;
  while (c > 0) {
    gotox(x + s);

    if (cbm_write(15, "b-p:2 0\n", 8) < 0) {
      printf("\nb-p failed: %d\n", _oserror);
      cbm_close(2);
      cbm_close(15);
      return(ERROR);
    }

    if ((l = cbm_write(2, 256*s + buffer, 256)) < 0) {
      col = textcolor(COLOR_RED);
      putchar('0' + (unsigned char) _oserror);
      textcolor(col);
    } else if (l != 256) {
      col = textcolor(COLOR_RED);
      putchar('W');
      textcolor(col);
    } else {
      putchar('W');
    }

    sprintf(cmd, "u2:2 0 %d %d\n", track, s);
    cbm_write(15, cmd, strlen(cmd));

    written[s] = 1;
    if (--c > 0) {
      s = (s + writeinterleave(drivetype, track)) % blockcount;
      while (written[s]) {
	s = (s + 1) % blockcount;
      }
    }
  }

  cbm_close(2);
  cbm_close(15);

  gotox(x + blockcount);

  return(OK);
}


/*

Verify a track on the drive

*/
unsigned char verifytrack(unsigned char device, unsigned char *buffer, unsigned char track, unsigned char blockcount, unsigned char drivetype) {
  unsigned char e = 0;
  unsigned char s, c, x, col;
  int l;
  unsigned char cmd[16];
  unsigned char verified[MAXBLOCKS];
  static unsigned char block[256];

  x = wherex();

  for (s = 0; s < MAXBLOCKS; ++s) {
    verified[s] = 0;
  }

  c = blockcount;

  if ((s = cbm_open(15, device, 15, "")) != 0) {
    printf("\nopen15,%d,15 failed: %d\n", device, s);
    return(ERROR);
  }
  if ((s = cbm_open(2, device, 2, "#")) != 0) {
    printf("\nopen2,%d,2,\"#\" failed: %d\n", device, s);
    cbm_close(15);
    return(ERROR);
  }

  s = 0;
  while (c > 0) {
    gotox(x + s);

    sprintf(cmd, "u1:2 0 %d %d\n", track, s);
    if (cbm_write(15, cmd, strlen(cmd)) < 0) {
      printf("\nu1 command failed: %d\n", _oserror);
      cbm_close(2);
      cbm_close(15);
      return(ERROR);
    }

    if (cbm_write(15, "b-p:2 0\n", 8) < 0) {
      printf("\nb-p failed: %d\n", _oserror);
      cbm_close(2);
      cbm_close(15);
      return(ERROR);
    }

    ++e;
    if ((l = cbm_read(2, block, 256)) < 0) {
      col = textcolor(COLOR_RED);
      putchar('0' + (unsigned char) _oserror);
      textcolor(col);
    } else if (l != 256) {
      col = textcolor(COLOR_YELLOW);
      putchar('V');
      textcolor(col);
    } else {
      l = 0;
      while (buffer[s*256 + l] == block[l] && l < 256) {
	++l;
      }
      if (l == 256) {
	putchar('V');
	--e;
      } else {
	col = textcolor(COLOR_RED);
	putchar('V');
	textcolor(col);
      }
    }

    verified[s] = 1;
    if (--c > 0) {
      s = (s + readinterleave(drivetype, track)) % blockcount;
      while (verified[s]) {
	s = (s + 1) % blockcount;
      }
    }
  }

  cbm_close(2);
  cbm_close(15);

  gotox(x + blockcount);

  if (e == 0) {
    return(OK);
  } else {
    return(ERROR);
  }
}


/*

Scan for drives

*/
iecdrive *scandrives() {
  unsigned char errchan[40];
  unsigned char d;
  unsigned char *s;
  iecdrive *newdrv;
  iecdrive *drv;

  drv = NULL;
  for (d = 8; d <= 30; ++d) {
    if (cbm_open(15, d, 15, "ui\n") == 0) {
      if (newdrv = malloc(sizeof(iecdrive))) {
	newdrv->prev = drv;
	newdrv->next = NULL;
	newdrv->dev = d;
	newdrv->type = DOTHER;
	cbm_read(15, errchan, 39);
	if (strncmp(errchan, "73,", 3) == 0) {
	  s = strchr(errchan + 3, ',') - 4;
	  if (strncmp(s, "1541", 4) == 0) {
	    newdrv->type = D1541;
	  } else if (strncmp(s, "1571", 4) == 0) {
	    newdrv->type = D1571;
	  } else if (strncmp(s, "1581", 4) == 0) {
	    newdrv->type = D1581;
	  }
	}

	if (drv) {
	  drv->next = newdrv;
	}
	drv = newdrv;
      }
    }
    cbm_close(15); 
  }
  if (drv) {
    while (drv->prev) {
      drv = drv->prev;
    }
  }
  return(drv);
}


/*

Send I0 to re-read BAM

*/
unsigned char sendi0(unsigned char device) {
  cbm_open(15, device, 15, "");
  if (cbm_write(15, "i0:\n", 4) < 0) {
    cbm_close(15);
    return(ERROR);
  }
  cbm_close(15);
  return(OK);
}


/*

Load directory

*/
unsigned int getiecdir(unsigned char *buffer, unsigned char device) {
  unsigned int length;

  buffer[0] = 0x01;
  buffer[1] = 0x04;
  length = cbm_load("$", device, buffer+ 2);
  if (length == 0) {
    printf("I/O error %d\n", _oserror);
  }
  return(length);
}
