/*

Drive geometry tables

*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "ssdisk.h"
#include "geometry.h"


static unsigned char g1541[] = {
  /* 35 tracks */
  35,
  /* 21 sectors on track 1-17*/
  21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
  /* 19 sectors on track 18-24*/
  19,19,19,19,19,19,19,
  /* 18 sectors on track 25-30*/
  18,18,18,18,18,18,
  /* 17 sectors on track 31-35*/
  17,17,17,17,17
};

static unsigned char interleave1541[] = {
  35,
  9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
  8,8,8,8,8,8,8,
  8,8,8,8,8,8,
  8,8,8,8,8
};

/*

Interleave benchmarks

Track 1, 1571 + C64 + SuperCPU

  7  19.0
  8  19.1
  9  16.3
 10  16.4
 11  16.5

Track 18, 1571 + C64 + SuperCPU

  6  17.7
  7  17.9
  8   8.9
  9   9.1
  10  9.7

Track 25, 1571 + C64 + SuperCPU

  7  17.5
  8   8.9

Track 21, 1571 + C64 + SuperCPU

  7  17.2
  8   8.4

*/

static unsigned char g1571[] = {
  70,
  21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
  19,19,19,19,19,19,19,
  18,18,18,18,18,18,
  17,17,17,17,17,
  21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,
  19,19,19,19,19,19,19,
  18,18,18,18,18,18,
  17,17,17,17,17
};

static unsigned char g1581[] = {
  80,
  40,40,40,40,40,40,40,40,40,40,
  40,40,40,40,40,40,40,40,40,40,
  40,40,40,40,40,40,40,40,40,40,
  40,40,40,40,40,40,40,40,40,40,
  40,40,40,40,40,40,40,40,40,40,
  40,40,40,40,40,40,40,40,40,40,
  40,40,40,40,40,40,40,40,40,40,
  40,40,40,40,40,40,40,40,40,40
};


unsigned int *g1541offset;
unsigned int *g1571offset;
unsigned int *g1581offset;


/*

Calculate offset tables

*/
unsigned char setupgeometry (void) {
  unsigned char t;
  unsigned int _int_;

  /* 1541 */
  if ((g1541offset = malloc(sizeof(_int_) * (g1541[0]+1))) == NULL) {
    return ERROR;
  }

  g1541offset[1] = 0;
  for (t = 2; t <= g1541[0]; ++t) {
    g1541offset[t] = g1541offset[t-1] + g1541[t-1];
  }

  /* 1571 */
  if ((g1571offset = malloc(sizeof(_int_) * (g1571[0]+1))) == NULL) {
    return ERROR;
  }
  
  g1571offset[1] = 0;

  for (t = 2; t <= g1571[0]; ++t) {
    g1571offset[t] = g1571offset[t-1] + g1571[t-1];
  }

  /* 1581 */
  if ((g1581offset = malloc(sizeof(_int_) * (g1581[0]+1))) == NULL) {
    return ERROR;
  }

  g1581offset[1] = 0;
  for (t = 2; t <= g1581[0]; ++t) {
    g1581offset[t] = g1581offset[t-1] + g1581[t-1];
  }

  return(OK);
}


/*

Return optimal interleave for a certain track

*/
unsigned char writeinterleave(unsigned char drivetype, unsigned char track) {
  if (drivetype == D1541) {
    return(interleave1541[track]);
  } else if (drivetype == D1571) {
    return(8);
  } else if (drivetype == D1581) {
    return(1);
  } else {
    return(1);
  }
}
unsigned char readinterleave(unsigned char drivetype, unsigned char track) {
  track += 0;
  if (drivetype == D1541) {
    return(4);
  } else if (drivetype == D1571) {
    return(4);
  } else if (drivetype == D1581) {
    return(1);
  } else {
    return(1);
  }
}


/*

Determine disk image type

*/
unsigned char imagetype(unsigned char *name, unsigned int size) {
  if (strlen(name) < 5) {
    return(DOTHER);
  }

  if (name[strlen(name) - 3] == 'd' || name[strlen(name) - 3] == 'D') {
    if (name[strlen(name) - 2] == '6' && name[strlen(name) - 1] == '4') {
      if (size == 689 || size == 692 || size == 775 || size == 778) {
	return(D1541);
      }
    } else if (name[strlen(name) - 1] == '1') {
      if (name[strlen(name) - 2] == '7') {
	if (size == 1378 || size == 1383) {
	  return(D1571);
	}
      } else if(name[strlen(name) - 2] == '8') {
	if (size == 3226 || size == 3238) {
	  return(D1581);
	}
      }
    }
  }

  return(DOTHER);
}
