/*

Serial port handling

gpz fixed 20020828: changed some references from RS_BAUD... to SER_...
					changed some references from RS_... to SS_...
					completed some void declarations (be a good boy mkay? ;=P)
	added 20020828: ser_flush

added comment on funcion ptrs at the bottom :)

*/


#include "ssdisk.h"
#include "serial.h"
#include <rs232.h>
#include "ss232.h"
#include <stdio.h>


/* translate Turbo232 baudrates to Silver Surfer baudrates */
#ifdef __C64__
static unsigned char transbaud[] = {
  0,0,0,0,0,0, /* 0, */ // gpz fixed 20020828
  SS_BAUD_2400,    /* SER_2400   0x06 RS_BAUD_2400   	0x06 SS_BAUD_2400    0x60 */
  SS_BAUD_4800,    /* SER_4800   0x07 RS_BAUD_4800   	0x07 SS_BAUD_4800    0x70 */
  SS_BAUD_9600,    /* SER_9600   0x08 RS_BAUD_9600   	0x08 SS_BAUD_9600    0x80 */
  SS_BAUD_19200,   /* SER_19200  0x09 RS_BAUD_19200  	0x09 SS_BAUD_19200   0x90 */
  SS_BAUD_38400,   /* SER_38400  0x0A RS_BAUD_38400  	0x0A SS_BAUD_38400   0xa0 */
  SS_BAUD_57600,   /* SER_57600  0x0B RS_BAUD_57600  	0x0B SS_BAUD_57600   0xb0 */
  SS_BAUD_115200,  /* SER_115200 0x0C RS_BAUD_115200 	0x0C SS_BAUD_115200  0xc0 */
  SS_BAUD_230400   /* SER_230400 0x0D RS_BAUD_230400 	0x0D SS_BAUD_230400  0xd0 */
};
#endif

/* pretty error messages */
static unsigned char *errormsg[] = {
  "",
  "Module not initialized",
  "Cannot handle baud rate",
  "Baud rate not available",
  "",
  ""
};


unsigned char activeinterface;


/* initialize the serial port */
unsigned char ser_init(unsigned char
#ifdef __C64__
		                     interface
#endif
		                              , unsigned char baudrate) {
  unsigned char status;

  if (baudrate < SER_2400 || baudrate > SER_230400) {
    printf("Couldn't set RS-232 parameters: %s", errormsg[RS_ERR_BAUD_NOT_AVAIL]);
    return(ERROR);
  }

#ifdef __C64__
  activeinterface = interface;
#ifdef DEBUG
  printf("using interface #%d\n", interface);
#endif
#endif

#ifdef __C64__
  if (interface == SER_TURBO232) {
#endif

    if ((status = rs232_init(0)) != RS_ERR_OK) {
      printf("Couldn't init RS-232: %s\n", errormsg[status]);
      return(ERROR);
    }

#ifdef DEBUG
    printf("init OK\n");
#endif

    if ((status = rs232_params(baudrate | RS_BITS_8, RS_PAR_NONE)) != RS_ERR_OK) {
      printf("Couldn't set RS-232 paramters: %s\n", errormsg[status]);
      (void) rs232_done();
      return(ERROR);
    }

#ifdef DEBUG
    printf("param OK\n");
#endif

#ifdef __C64__
  } else {

    if ((status = ss232_init(0)) != SS_ERR_OK) {
      printf("Couldn't init RS-232: %s\n", errormsg[status]);
      return(ERROR);
    }

    if ((status = ss232_params(transbaud[baudrate] | SS_BITS_8, SS_PAR_NONE)) != SS_ERR_OK) {
      printf("Couldn't set RS-232 paramters: %s\n", errormsg[status]);
//      (void) rs232_done();
      (void) ss232_done();  // gpz fixed 20020828
      return(ERROR);
    }

  }
#endif

  return(OK);
}


/* call when done with the serial port */
void ser_exit(void) {
#ifdef __C64__
  if (activeinterface == SER_TURBO232) {
#endif
    (void) rs232_done();
#ifdef __C64__
  } else {
    (void) ss232_done();
  }
#endif
}


/* get a byte from the serial port, blocking */
unsigned char ser_rx(void) {
  unsigned char c;

#ifdef __C64__
  if (activeinterface == SER_TURBO232) {
#endif
    while (rs232_get(&c) != RS_ERR_OK) { ; }
#ifdef __C64__
  } else {
    while (ss232_get(&c) != SS_ERR_OK) { ; }
  }
#endif

  return(c);
}


/* send a byte on the serial port */
void ser_tx(unsigned char byte) {
#ifdef __C64__
  if (activeinterface == SER_TURBO232) {
#endif
    while (rs232_put(byte) != RS_ERR_OK) { ; }
#ifdef __C64__
  } else {
    while (ss232_put(byte) != SS_ERR_OK) { ; }
  }
#endif
}
 
 
/* get an LSB first 16-bit word */
unsigned int ser_getword(void) {
  unsigned int w;

  w = ser_rx();
  w = w + ser_rx()*256;
  return(w);
}

/* send a string on the serial port */
void ser_send(unsigned char *string, unsigned char length) {
  unsigned int c;

  for (c = 0; c < length; ++c) {
    ser_tx(string[c]);
  }
}

/* read until no more incoming data presnt */
void ser_flush(void) {
unsigned short out;
unsigned char c;

	out=0;

#ifdef __C64__
  if (activeinterface == SER_TURBO232) {
#endif
    while (rs232_get(&c) == RS_ERR_OK) {
		++out; if(out==0xffff) break;
	#ifdef __C64__
		*((char*)0xd020)=*((char*)0xd020)+1;
		*((char*)0xd020)=*((char*)0xd020)-1;
	#endif
		}
#ifdef __C64__
  } else {
    while (ss232_get(&c) == SS_ERR_OK) {
		++out; if(out==0xffff) break;
		#ifdef __C64__
			*((char*)0xd020)=*((char*)0xd020)+1;
			*((char*)0xd020)=*((char*)0xd020)-1;
		#endif
		}
  }
#endif
}

/*

<MagerValp> is there any way of redirecting a function call at runtime
<WhiteFlam> Function pointer
<MagerValp> k
<MagerValp> how? :)
<WhiteFlam> typdefs make it easier:
<WhiteFlam> typedef void (*funcCall)(params...)
<WhiteFlam> (I think.  The '*' might go after the var name)
<MagerValp> hrm
<MagerValp> doesn't exactly fall into place here
<WhiteFlam> Whatcha doing anyway?
<MagerValp> i have two functions, rs232_rx and ss232_rx. i wish ser_rx to call
            one of them
<WhiteFlam> if(blah) rs232_rx(); else ss232_rx();
<WhiteFlam> ;)
<MagerValp> right now it's if (activeinterface == TURBO232) { rs232_rx() }
            else { ss232_rx() }
<MagerValp> yeah
<MagerValp> but that's too much overhead
<WhiteFlam> is it?  What CPU are you running?
<MagerValp> 6502 of course
<WhiteFlam> okay, then it is too much overhead. :)
<MagerValp> i'll probably implement these in assembler eventually, but until
            then
<WhiteFlam> okay, get that typedef from above.
<WhiteFlam> funcCall foo;
<MagerValp> ack
<WhiteFlam> foo = rs232_rx;
<MagerValp> literally type )(params...) ?
<WhiteFlam> Then, do (*foo)() to do the function call.
<WhiteFlam> no, params... is your parameter list
<MagerValp> k
<WhiteFlam> A function pointer includes parameters and a return type
<MagerValp> and (*ser_rx)(byte) to call ?
<WhiteFlam> yes, if you've got "funcCall ser_rx;"
<MagerValp> ok
<MagerValp> no way of getting rid of (*) ?
<WhiteFlam> It's a pointer.  You have to dereference it.
<MagerValp> ok


loooook, from some cpu-core....

void cmd_fb(void) {
}

...etc...

volatile void *commands[0x100] = {
    &cmd_00,&cmd_01,&cmd_na,&cmd_03,&cmd_82,&cmd_05,&cmd_06,&cmd_07,&cmd_08,&cmd_09,&cmd_0a,&cmd_0b,&cmd_0c,&cmd_0d,&cmd_0e,&cmd_0f,
    &cmd_10,&cmd_11,&cmd_na,&cmd_13,&cmd_82,&cmd_15,&cmd_16,&cmd_17,&cmd_18,&cmd_19,&cmd_ea,&cmd_1b,&cmd_0c,&cmd_1d,&cmd_1e,&cmd_1f,
    &cmd_20,&cmd_21,&cmd_na,&cmd_23,&cmd_24,&cmd_25,&cmd_26,&cmd_27,&cmd_28,&cmd_29,&cmd_2a,&cmd_2b,&cmd_2c,&cmd_2d,&cmd_2e,&cmd_2f,
    &cmd_30,&cmd_31,&cmd_na,&cmd_33,&cmd_82,&cmd_35,&cmd_36,&cmd_37,&cmd_38,&cmd_39,&cmd_xx,&cmd_3b,&cmd_0c,&cmd_3d,&cmd_3e,&cmd_3f,
    &cmd_40,&cmd_41,&cmd_na,&cmd_xx,&cmd_82,&cmd_45,&cmd_46,&cmd_47,&cmd_48,&cmd_49,&cmd_4a,&cmd_xx,&cmd_4c,&cmd_4d,&cmd_4e,&cmd_xx,
    &cmd_50,&cmd_51,&cmd_na,&cmd_xx,&cmd_82,&cmd_55,&cmd_56,&cmd_57,&cmd_58,&cmd_59,&cmd_ea,&cmd_xx,&cmd_0c,&cmd_5d,&cmd_5e,&cmd_5f,
    &cmd_60,&cmd_61,&cmd_na,&cmd_xx,&cmd_82,&cmd_65,&cmd_66,&cmd_67,&cmd_68,&cmd_69,&cmd_6a,&cmd_xx,&cmd_6c,&cmd_6d,&cmd_6e,&cmd_xx,
    &cmd_70,&cmd_71,&cmd_na,&cmd_xx,&cmd_82,&cmd_75,&cmd_76,&cmd_xx,&cmd_78,&cmd_79,&cmd_xx,&cmd_xx,&cmd_0c,&cmd_7d,&cmd_7e,&cmd_7f,
    &cmd_80,&cmd_81,&cmd_82,&cmd_xx,&cmd_84,&cmd_85,&cmd_86,&cmd_87,&cmd_88,&cmd_82,&cmd_8a,&cmd_xx,&cmd_8c,&cmd_8d,&cmd_8e,&cmd_xx,
    &cmd_90,&cmd_91,&cmd_na,&cmd_xx,&cmd_94,&cmd_95,&cmd_96,&cmd_xx,&cmd_98,&cmd_99,&cmd_9a,&cmd_xx,&cmd_xx,&cmd_9d,&cmd_xx,&cmd_xx,
    &cmd_a0,&cmd_a1,&cmd_a2,&cmd_xx,&cmd_a4,&cmd_a5,&cmd_a6,&cmd_xx,&cmd_a8,&cmd_a9,&cmd_aa,&cmd_xx,&cmd_ac,&cmd_ad,&cmd_ae,&cmd_af,
    &cmd_b0,&cmd_b1,&cmd_na,&cmd_xx,&cmd_b4,&cmd_b5,&cmd_b6,&cmd_xx,&cmd_b8,&cmd_b9,&cmd_ba,&cmd_xx,&cmd_bc,&cmd_bd,&cmd_be,&cmd_bf,
    &cmd_c0,&cmd_c1,&cmd_82,&cmd_c3,&cmd_c4,&cmd_c5,&cmd_c6,&cmd_c7,&cmd_c8,&cmd_c9,&cmd_ca,&cmd_xx,&cmd_cc,&cmd_cd,&cmd_ce,&cmd_xx,
    &cmd_d0,&cmd_d1,&cmd_na,&cmd_d3,&cmd_82,&cmd_d5,&cmd_d6,&cmd_xx,&cmd_d8,&cmd_d9,&cmd_xx,&cmd_xx,&cmd_0c,&cmd_dd,&cmd_de,&cmd_xx,
    &cmd_e0,&cmd_e1,&cmd_82,&cmd_e3,&cmd_e4,&cmd_e5,&cmd_e6,&cmd_e7,&cmd_e8,&cmd_e9,&cmd_ea,&cmd_xx,&cmd_ec,&cmd_ed,&cmd_ee,&cmd_ef,
    &cmd_f0,&cmd_f1,&cmd_na,&cmd_f3,&cmd_82,&cmd_f5,&cmd_f6,&cmd_f7,&cmd_f8,&cmd_f9,&cmd_ea,&cmd_fb,&cmd_0c,&cmd_fd,&cmd_fe,&cmd_ff,
};


for calling:

((void(*)())commands[curr_inst])();

maybe helps :o)


*/

