/*
* ss232.h
*
* Groepaz/Hitmen, 16.12.2001
*
* This defines for the SilverSurver (16c550 UART) what Ullrichs rs232 module
* defines for the Swithlink/Turbo232
*
* this driver operates in polling mode only atm !
*
* rs232.h conflicts eliminated by Per Olofsson, 2002-07-25
*/



#ifndef _SS232_H
#define _SS232_H



/*****************************************************************************/
/*                   		    Data                                     */
/*****************************************************************************/


/* Baudrate settings */
#define SS_BAUD_50                0x00
#define SS_BAUD_110               0x10
#define SS_BAUD_134_5             0x20
#define SS_BAUD_300               0x30
#define SS_BAUD_600               0x40
#define SS_BAUD_1200              0x50
#define SS_BAUD_2400              0x60
#define SS_BAUD_4800              0x70
#define SS_BAUD_9600              0x80
#define SS_BAUD_19200             0x90
#define SS_BAUD_38400             0xa0
#define SS_BAUD_57600             0xb0
#define SS_BAUD_115200            0xc0
#define SS_BAUD_230400            0xd0

/* Parity settings */
#define SS_PAR_NONE                     0x00
#define SS_PAR_ODD                      0x28
#define SS_PAR_EVEN                     0x38
#define SS_PAR_MARK                     0x48
#define SS_PAR_SPACE                    0x48

/* Data bit settings */
#define SS_BITS_5                       0x00
#define SS_BITS_6                       0x01
#define SS_BITS_7                       0x02
#define SS_BITS_8                       0x03

/* Bit masks to mask out things from the status returned by rs232_status */
#define SS_STATUS_IRQ                   0x01    /* (iir) IRQ condition */
#define SS_STATUS_OVERRUN               0x02    /* (lsr) Overrun error */
#define SS_STATUS_PE                    0x04    /* (lsr) Parity error */
#define SS_STATUS_FE                    0x08    /* (lsr) Framing error */
#define SS_STATUS_DSR                   0x10    /* (msr>>1) NOT data set ready */
#define SS_STATUS_THRE                  0x20    /* (lsr) Transmit holding reg. empty */
#define SS_STATUS_DCD                   0x40    /* (msr>>1) NOT data carrier detect */
#define SS_STATUS_RDRF                  0x80    /* Receiver data register full */

/* Stop bit settings */
#define SS_STOP_1                       0x00
#define SS_STOP_2                       0x04

/* Error codes returned by all functions */
#define SS_ERR_OK                       0x00    /* Not an error - relax */
#define SS_ERR_NOT_INITIALIZED          0x01    /* Module not initialized */
#define SS_ERR_BAUD_TOO_FAST            0x02    /* Cannot handle baud rate */
#define SS_ERR_BAUD_NOT_AVAIL           0x03    /* Baud rate not available */
#define SS_ERR_NO_DATA                  0x04    /* Nothing to read */
#define SS_ERR_OVERFLOW                 0x05    /* No room in send buffer */



/*****************************************************************************/
/*                                   Code                                    */
/*****************************************************************************/



unsigned char __fastcall__ ss232_init (char hacked);
/* Initialize the serial port, install the interrupt handler. The parameter
* has no effect for now and should be set to 0.
*/

unsigned char __fastcall__ ss232_params (unsigned char params, unsigned char parity);
/* Set the port parameters. Use a combination of the #defined values above. */

unsigned char __fastcall__ ss232_done (void);
/* Close the port, deinstall the interrupt hander. You MUST call this function
* before terminating the program, otherwise the machine may crash later. If
* in doubt, install an exit handler using atexit(). The function will do
* nothing, if it was already called.
*/

unsigned char __fastcall__ ss232_get (char* b);
/* Get a character from the serial port. If no characters are available, the
* function will return SS_ERR_NO_DATA, so this is not a fatal error.
*/

unsigned char __fastcall__ ss232_put (char b);
/* Send a character via the serial port. There is a transmit buffer, but
* transmitting is not done via interrupt. The function returns
* SS_ERR_OVERFLOW if there is no space left in the transmit buffer.
*/

unsigned char __fastcall__ ss232_pause (void);
/* Assert flow control and disable interrupts. */

unsigned char __fastcall__ ss232_unpause (void);
/* Re-enable interrupts and release flow control */

unsigned char __fastcall__ ss232_status (unsigned char* status,
                                         unsigned char* errors);
/* Return the serial port status. */



/* End of ss232.h */
#endif
