/*

Interact with SS server

gpz	added 20020828: ser_flush calls before commands, makes interacting much
					more convinient.

*/


#include <string.h>
#include <stdio.h>
#include <conio.h>
#include <rs232.h>
#include <errno.h>

#include "ssdisk.h"
#include "sscmd.h"
#include "geometry.h"
#include "serial.h"


static unsigned char blcommand[] = { 'b', 'l' , '%', 's', 0x0d, 0x0a, 0x00 };
       unsigned char brcommand[] = { 'b', 'r' , 0x00, 0x00, 0x00, 0x0d, 0x0a, 0x00 };


/*

Buffer a file for loading
Returns number of buffered blocks

*/
unsigned int blockload(unsigned char *filename) {
  unsigned char length;
  unsigned char cmdstring[40];

  length = strlen(filename);
  if (length > 0) {
    sprintf(&cmdstring, blcommand, filename);

  	ser_flush();

    ser_send(cmdstring, length + 4);

    if (ser_rx() != '!') {
      return(0);
    }

    return(ser_getword());
  }

  return(0);
}


/*

Read buffered blocks from SS

*/
unsigned char readblocks(unsigned char *buffer, unsigned int startblock, unsigned char blockcount) {
  unsigned int bptr = 0;
  unsigned char b = 0;
  unsigned char status, l, col;

  brcommand[2] = startblock & 255;
  brcommand[3] = startblock >> 8;
  brcommand[4] = blockcount;

  ser_flush();

  ser_send(brcommand, 7);

  if (ser_rx() != '!') {
    return(ERROR);
  }

  while (b < blockcount) {
    if (ser_rx() != 255) { return(ERROR); }

    l = 0;
    do {
      buffer[bptr++] = ser_rx();
    } while (l++ != 255);

    cksum[b] = ser_rx();

    ++b;
  }

  for (b = 0; b < blockcount; ++b) {
    if (checksum(buffer + 256*b) == cksum[b]) {
      putchar('R');
    } else {
      col = textcolor(COLOR_RED);
      putchar('R');
      textcolor(col);
      gotox(wherex() - 1);
      status = readblocks(buffer + 256*b, startblock + b, 1);
    }
  }

  return(OK);
}


/*

Calculate sum of all bytes mod 256 checksum of a 256-byte block

*/
unsigned char checksum(unsigned char *block) {
  unsigned char l = 0;
  unsigned char c = 0;

  c = 0;
  l = 0;
  do {
    c += block[l++];
  } while (l != 0);

  return(c);
}


/*

Load directory from SS

*/
unsigned int getssdir(unsigned char *buffer) {
  unsigned int length;
  unsigned int bptr;

  ser_flush();

  ser_send("<$\n\r", 4);

  if (ser_rx() != '!') {
    return(0);
  }

  length = ser_getword();

  for (bptr = 0; bptr < length; ++bptr) {
    buffer[bptr] = ser_rx();
  }

  return(length);
}


/*

Ping server and get version string

*/
unsigned char pingss(unsigned char *ssver) {
  unsigned int length;
  unsigned int i;

  ser_flush();

  ser_send("?\n\r", 3);

  if (ser_rx() != '!') {
    return(ERROR);
  }

  length = ser_getword();

  for (i = 0; i < length; ++i) {
    ssver[i] = ser_rx();
  }
  ssver[i] = 0;

  return(OK);
}


/*

Change directory

*/
unsigned char changessdir(unsigned char *name) {

  ser_flush();

  ser_tx('<');
  ser_send(name, strlen(name));
  ser_send("\n\r", 2);

  if (ser_rx() != '.') {
    return(ERROR);
  }
  return(OK);
}
