;---------------------------------------
; Serial Slave file copy
;
; File copy utility for Serial Slave
;
; History
;  0.1  copy from IEC to SS
;  0.2  major bugfixes
;  0.3  fixed short dir listing
;       copied file-1 bytes, fixed
;       prints file size when copying
;  0.4  support for 16550, and prepared
;       code for multiple targets
;       copy from SS to IEC
;  0.5  better filename and type recognition
;       directory scrolling sped up
;       added select all, none, inverse
;       added reread function
;  0.6  ported to the C128 
;       fixed copying past end of list
;       fixed a, n, i not updating nsel
;       swapped out basic roms on C64 and C128
;       maxentries changed to 144
;       added status line
;  0.7  rewrote most of the screen handling
;       kernal getin input handling
;       ported to the Plus/4


	include "ssconst.i"


	processor 6502


dev	= 6

	include "kernal.i"

maxlines	= 20

; ascii codes
revon	= 18
revoff	= 146
home	= 19
clr	= 147


	include "platform.i"


;---------------------------------------
; INIT

init	subroutine
 ifconst C64
	lda #$26	; kernal, I/O
	sta $01
 endif

 ifconst C128
	lda #%00001110	; kernal, I/O, ram
	sta $ff00
 endif
	jsr mkasctab


	jsr scrorg	; get screen geometry
 ifconst C128
	tax	; C128 returns screen width -1 in A
	inx
 endif
	stx linelen

 ifconst C128
	cpx #80	; 80 col mode?
	bne .not80
		; set VDC colours
	ldx #$1a	; light grey background
	lda #14
	jsr stv

	jmp .skip40
.not80
 endif

; set VIC/TED colours
 ifnconst PET
	lda #bkgcol	; medium grey screen
	sta sbkg
	lda #0	; black border
	sta sborder
 endif

.skip40

 ifconst C128
	jsr setfullwin	; in case we started with a window defined
 endif

	lda fa	; default device
	cmp #dev
	bne .notss
	lda #8
.notss
	sta iecdev
	lda #0	; copy from SS
	sta dir

	jsr ptitle
	jsr getdir
	jsr printlist


;---------------------------------------
; MAIN

main	jsr waitkey

	cmp #"D"
	bne notdev
	jsr nextdev
	jmp main
notdev
	cmp #17	; crsr down
	bne notdown
	jsr down
	jmp main
notdown
	cmp #145	; crsr up
	bne notup
	jsr up
	jmp main
notup
	cmp #95	; <-
	bne notswap
	jsr swap
	jmp main
notswap
	cmp #" "
	bne notselct
	jsr select
	jmp main
notselct
	cmp #13	; enter
	bne notstart
	jsr start
	jmp main

notstart
	cmp #"Q"
	bne notquit
quit	lda #147
	jsr bsout

 ifconst C64
	lda #$27
	sta $01
 endif

 ifconst C128
	lda #%00000000
	sta $ff00
 endif

	rts
notquit
	cmp #20
	bne notparen
	jsr cdparen
	jmp main

notparen
	cmp #19
	bne nothome
	jsr gohome
	jmp main

nothome
	cmp #"R"
	bne notreread
	jsr reread
	jmp main

notreread
	cmp #"A"
	bne notall
	jsr selall
	jmp main

notall
	cmp #"N"
	bne notnone
	jsr selnone
	jmp main

notnone
	cmp #"I"
	bne notinverse
	jsr selinverse
	jmp main

notinverse
 ifnconst PET
	inc sborder
	ldx rastline
	dex
waitvblx	cpx rastline
	bne waitvblx
	dec sborder
 endif
	jmp main

	rts


;---------------------------------------
; BSS

dir	dc.b 0
iecdev	dc.b 0
savey	dc.b 0
savex	dc.b 0
nfiles	dc.b 0
curline	dc.b 0
offset	dc.b 0
nsel	dc.b 0
linelen	dc.b 0


;---------------------------------------
; DATA

dollar	dc.b "$"

titlescr	; clr, lc, black
	dc.b 147, 14, 144
	; rvs on, ..., rvs off
	dc.b revon, "sERIAL sLAVE FILE COPY UTILITY V0.7 2002", revoff
	dc.b 19, 17
	dc.b 5, "CRSR", 151, " AND ", 5, "SPACE", 151, " SELECTS, ", 5, "rETURN", 151, " STARTS", 13
	dc.b 5, 95, 151, " TO CHANGE TO/FROM ss, ", 5, "del", 151, " DOES CD ..  "
	dc.b 0

fromss	; home, down, white
	dc.b 19,17,17,17
	dc.b "cURRENT DEST ", 5, "d", 151, "RIVE:     "
	; left, left
	dc.b 157,157,0

toss	; home, down
	dc.b 19,17,17,17
	dc.b "cURRENT SOURCE ", 5, "d", 151, "RIVE:   "
	; left, left
	dc.b 157,157,0


;---------------------------------------
; Subs


;---------------------------------------
; clear tables

clrtables	subroutine
	lda #0
	sta nsel
	sta nfiles
	sta curline
	sta offset
	tax
.loop	sta ntmp,x
	sta sizel,x
	sta sizem,x
	sta type,x
	sta sel,x
	inx
	cpx #maxentries
	bne .loop
	rts


;---------------------------------------
; clear all selections

clrsel	lda #0
	sta nsel
	tax
clrsellp	sta sel,x
	inx
	cpx #maxentries
	bne clrsellp
	rts


;---------------------------------------
; set all select flags

setsel	ldx #0
	stx nsel
setnext	ldy #0
	lda type,x
	cmp #"P"
	bne setnotprg
	ldy #1
	inc nsel
setnotprg	tya
	sta sel,x
	inx
	cpx nfiles
	bne setnext
	rts


;---------------------------------------
; print title

ptitle	ldx #0
	lda #<titlescr
	ldx #>titlescr
	jsr pstring
	lda #<fromss
	ldx #>fromss
	ldy dir
	beq pttoss
	lda #<toss
	ldx #>toss
pttoss
	jsr pstring
	ldx iecdev
	lda #0
	jsr print16
	lda #13
	jsr bsout
	rts


;---------------------------------------
; change to the next iec dev

nextdev	subroutine
	ldx iecdev
	inx
	txa
	and #7
	ora #8
	sta iecdev
	jsr ptitle
	lda dir	; no need to reload
	beq .done
	jsr getdir	; get iec dir
.done	jsr printlist
	rts


;---------------------------------------

swap	lda dir
	eor #1
	sta dir
	jsr ptitle
	jsr getdir
	jsr printlist
	rts


;---------------------------------------
; reread directory

reread	jsr getdir
	jsr printlist
	rts


;---------------------------------------
; move cursor down

down	subroutine
	lda nfiles	; can't move if there aren't any files
	beq .done
	lda curline	; cl+of = nfiles?
	sec
	adc offset
	cmp nfiles
	bne .notdmax
	rts
.notdmax
	lda curline
	cmp #maxlines-1
	bne .notdbotm
	inc offset
	jsr scrollup
	lda #19
	jsr printline
	rts
.notdbotm
	lda #" "
	jsr printcsr
	inc curline
	lda #">"
	jsr printcsr
.done	rts


;---------------------------------------
; move cursor up

up	subroutine
	lda nfiles	; can't move if there aren't any files
	beq .done
	lda curline
	bne .notltop
	lda offset
	beq .noupoffs
	dec offset
	jsr scrolldown
	lda #0
	jsr printline
.noupoffs
	rts
.notltop
	lda #" "
	jsr printcsr
	dec curline
	lda #">"
	jsr printcsr
.done	rts


;---------------------------------------
; move cursor home

gohome	subroutine
	lda nfiles	; can't move if there aren't any files
	bne .dohome
	rts
.dohome	lda #0
	sta curline
	sta offset
	jmp printlist


;---------------------------------------
; select all

selall	subroutine
	lda nfiles	; can't move if there aren't any files
	bne .doselall
	rts
.doselall	jsr setsel
	jmp printlist


;---------------------------------------
; select none

selnone	subroutine
	lda nfiles	; can't move if there aren't any files
	bne .doselnone
	rts
.doselnone	jsr clrsel
	jmp printlist


;---------------------------------------
; invert selection

selinverse	subroutine
	lda nfiles	; can't move if there aren't any files
	bne .doinverse
	rts
.doinverse	ldx #0
selnext	lda type,x
	cmp #"P"
	bne skipinv

	lda sel,x	; flip select bit
	eor #1
	sta sel,x

	tay	; inc nsel if we selected
	lda invtab,y	; dec nsel if we deselected
	clc
	adc nsel
	sta nsel
skipinv
	inx
	cpx nfiles
	bne selnext

	jmp printlist

invtab	dc.b -1,1


;---------------------------------------
; mark an entry for copying

select	subroutine
	lda nfiles	; can't select if there aren't any files
	beq .done
	lda curline
	clc
	adc offset
	tax
	lda type,x
	cmp #"D"
	beq changdir
	cmp #"P"
	beq .selp
	rts	; only prg supported
.selp	lda sel,x
	eor #1
	sta sel,x
	beq .decnsel
	inc nsel
.seldone	lda curline
	jsr printline
	jsr drawstatus
.done	rts
.decnsel	dec nsel
	jmp .seldone

; change current directory
changdir	subroutine
	lda dir	; only cd on ss
	bne chgddone
	txa	; save line nr
	jsr xsetnam
docd	jsr ssload
	jsr ptitle
	jsr getdir
	jsr printlist
chgddone	rts


;---------------------------------------
; cd ..

cdparen	lda dir
	bne chgddone
	lda #2
	ldx #<dotdot
	ldy #>dotdot
	jsr setnam
	jmp docd

dotdot	dc.b ".."


;---------------------------------------
; start the copying

start	subroutine
	lda nfiles
	bne .checknsel
	rts
.checknsel	lda nsel
	bne canstart
	rts
canstart	jsr clrlist

	ldx #4	; jump to 0,4
	ldy #0
	clc
	jsr plot

	ldx #0
nextfile	lda sel,x
	beq notcopme

	txa
	pha

	jsr xsetnam

	jsr primm
	dc.b "COPYING "
	dc.b 0

	pla
	pha
	jsr printnam

	lda dir
	bne cptoss
cpfromss
	jsr ssload
	bcs cpdone
	jsr iecsave
	bcs cperror
	bcc cpdone

cptoss	jsr iecload
	bcs cperror

	lda #" "
	jsr bsout

	lda eal	; print length
	sec
	sbc #<buffer
	tax
	lda eah
	sbc #>buffer
	pha
	txa	; +2 to compensate for load addr
	clc
	adc #2
	txa
	pla
	adc #0
	jsr print16

	lda #" "
	jsr bsout

	jsr save
	bcc cpdone

cperror	jsr primm
	dc.b " FAILED"
	dc.b 0

cpdone	lda #13
	jsr bsout
	pla
	tax
notcopme
	inx
	cpx nfiles
	bne nextfile

	jsr primm
	dc.b 13
	dc.b "PRESS SPACE TO CONTINUE"
	dc.b 0

.waitspcorq	jsr waitkey	; wait for space or q
	cmp #" "
	beq .gotspace
	cmp #"Q"
	bne .waitspcorq

	pla	; chew return address
	pla
	jmp quit

.gotspace	jsr clrsel
	lda #0
	sta offset	; reset cursor
	sta curline

	jsr ptitle	; redraw screen
	jsr printlist
	rts


;---------------------------------------
; print filename

printnam	pha
	lda #34	; \"
	jsr bsout

	pla	; get pointer to
	jsr name2tmp	; name in tmp

	ldy #0	; print name
pname	lda (tmp),y
	beq pnamedn
	jsr bsout
	iny
	cpy #16
	bne pname
pnamedn
	lda #34
	jsr bsout
	rts


;---------------------------------------
; call setnam for the file in A

xsetnam	pha
 ifconst C128
	lda #0
	tax
	jsr setbnk
	pla
	pha
 endif
	jsr name2tmp
	ldy #0
xsgetlen	lda (tmp),y
	beq xsgotlen
	iny
	cpy #16
	bne xsgetlen
xsgotlen	tya
	ldx tmp
	ldy tmp+1
	jsr setnam
	pla
	rts


;---------------------------------------
; print the directory listing

printlist	subroutine
	lda nfiles
	bne nfnot0
	rts
nfnot0
	jsr clrlist

	lda #0
	sta .line
.drawloop	lda .line
	jsr printline
	inc .line
	lda .line
	cmp nfiles
	beq .done
	cmp #20
	bne .drawloop
.done
	lda #">"
	jsr printcsr
	jmp drawstatus

.line	dc.b 0


;---------------------------------------
; draw status line

drawstatus	subroutine
	ldx #24	; at the bottom
	ldy #0
	clc
	jsr plot

	lda #144	; black
	jsr bsout

	ldx nsel	; number of selected files
	lda #0
	jsr print16

	lda #"/"	; /
	jsr bsout

	ldx nfiles	; number of total files
	lda #0
	jsr print16

	jsr primm
	dc.b " FILES SELECTED",0

	rts


;---------------------------------------
; print line A in dir list

printline	subroutine
	pha

	clc
	adc offset
	sta ol	; keep track of line+offset

	pla
	clc	; move cursor to line + 4
	adc #4
	tax
	ldy #2
	clc
	jsr plot	; set cursor

	ldy #revoff	; rev off
	ldx ol
	lda sel,x
	beq .notsel
	ldy #revon	; rev on
.notsel
	tya
	jsr bsout

	lda #144	; black
	jsr bsout

	ldx #23	; print 23 spaces
	lda #" "
clrline	jsr bsout
	dex
	bne clrline

	lda #1	; back to column 1
	jsr setcol

	ldx ol
	lda sizem,x
	pha
	lda sizel,x
	tax
	pla
	jsr print16	; print size

	lda #6	; jump to col 6
	jsr setcol

	lda ol	; print name
	jsr printnam

	lda #25	; jump to col 25
	jsr setcol

	ldx ol
	lda type,x	; print type
	jsr prntype

	lda #revoff	; in case rev is on
	jsr bsout

	rts


;---------------------------------------
; print A at cursor

printcsr	pha
	lda curline
	clc
	adc #4
	tax	; cursor to 0,4+curline
	ldy #0
	clc
	jsr plot	; set cursor

	lda #144	; black
	jsr bsout

	pla	; print character
	jsr bsout
	rts


;---------------------------------------
; scroll list up

 ifconst window
scrollup	jsr setlistwin
	jsr primm	; Esc-V to scroll up
	dc.b 27,"V",0
	jmp setfullwin
 else

scrollup	subroutine
 ifconst PET
	lda linelen	; handle 80 column mode scrolling
	cmp #80	; on pet
	beq .scup80col
 endif
	ldx #27
.scuploop	lda screenbase+5*40,x
	sta screenbase+4*40,x
	lda screenbase+6*40,x
	sta screenbase+5*40,x
	lda screenbase+7*40,x
	sta screenbase+6*40,x
	lda screenbase+8*40,x
	sta screenbase+7*40,x
	lda screenbase+9*40,x
	sta screenbase+8*40,x
	lda screenbase+10*40,x
	sta screenbase+9*40,x
	lda screenbase+11*40,x
	sta screenbase+10*40,x
	lda screenbase+12*40,x
	sta screenbase+11*40,x
	lda screenbase+13*40,x
	sta screenbase+12*40,x
	lda screenbase+14*40,x
	sta screenbase+13*40,x
	lda screenbase+15*40,x
	sta screenbase+14*40,x
	lda screenbase+16*40,x
	sta screenbase+15*40,x
	lda screenbase+17*40,x
	sta screenbase+16*40,x
	lda screenbase+18*40,x
	sta screenbase+17*40,x
	lda screenbase+19*40,x
	sta screenbase+18*40,x
	lda screenbase+20*40,x
	sta screenbase+19*40,x
	lda screenbase+21*40,x
	sta screenbase+20*40,x
	lda screenbase+22*40,x
	sta screenbase+21*40,x
	lda screenbase+23*40,x
	sta screenbase+22*40,x
	dex
	bne .scuploop
	rts

 ifconst PET
.scup80col	ldx #27
.scup80loop	lda screenbase+5*80,x	; fixme for 80-col mode
	sta screenbase+4*80,x
	lda screenbase+6*80,x
	sta screenbase+5*80,x
	lda screenbase+7*80,x
	sta screenbase+6*80,x
	lda screenbase+8*80,x
	sta screenbase+7*80,x
	lda screenbase+9*80,x
	sta screenbase+8*80,x
	lda screenbase+10*80,x
	sta screenbase+9*80,x
	lda screenbase+11*80,x
	sta screenbase+10*80,x
	lda screenbase+12*80,x
	sta screenbase+11*80,x
	lda screenbase+13*80,x
	sta screenbase+12*80,x
	lda screenbase+14*80,x
	sta screenbase+13*80,x
	lda screenbase+15*80,x
	sta screenbase+14*80,x
	lda screenbase+16*80,x
	sta screenbase+15*80,x
	lda screenbase+17*80,x
	sta screenbase+16*80,x
	lda screenbase+18*80,x
	sta screenbase+17*80,x
	lda screenbase+19*80,x
	sta screenbase+18*80,x
	lda screenbase+20*80,x
	sta screenbase+19*80,x
	lda screenbase+21*80,x
	sta screenbase+20*80,x
	lda screenbase+22*80,x
	sta screenbase+21*80,x
	lda screenbase+23*80,x
	sta screenbase+22*80,x
	dex
	bne .scup80loop
	rts
 endif

 endif	; ifconst C128


;---------------------------------------
; scroll list down

 ifconst window
scrolldown	jsr setlistwin
	jsr primm	; Esc-W to scroll down
	dc.b 27,"W",0
	jmp setfullwin
 else

scrolldown	subroutine
 ifconst PET
	lda linelen	; handle 80 column mode scrolling
	cmp #80	; on pet
	beq .scdn80col
 endif
	ldx #27
.scdnloop
	lda screenbase+22*40,x
	sta screenbase+23*40,x
	lda screenbase+21*40,x
	sta screenbase+22*40,x
	lda screenbase+20*40,x
	sta screenbase+21*40,x
	lda screenbase+19*40,x
	sta screenbase+20*40,x
	lda screenbase+18*40,x
	sta screenbase+19*40,x
	lda screenbase+17*40,x
	sta screenbase+18*40,x
	lda screenbase+16*40,x
	sta screenbase+17*40,x
	lda screenbase+15*40,x
	sta screenbase+16*40,x
	lda screenbase+14*40,x
	sta screenbase+15*40,x
	lda screenbase+13*40,x
	sta screenbase+14*40,x
	lda screenbase+12*40,x
	sta screenbase+13*40,x
	lda screenbase+11*40,x
	sta screenbase+12*40,x
	lda screenbase+10*40,x
	sta screenbase+11*40,x
	lda screenbase+9*40,x
	sta screenbase+10*40,x
	lda screenbase+8*40,x
	sta screenbase+9*40,x
	lda screenbase+7*40,x
	sta screenbase+8*40,x
	lda screenbase+6*40,x
	sta screenbase+7*40,x
	lda screenbase+5*40,x
	sta screenbase+6*40,x
	lda screenbase+4*40,x
	sta screenbase+5*40,x

	dex
	bne .scdnloop
	rts

 ifconst PET
.scdn80col	ldx #27
.scdn80loop
	lda screenbase+22*80,x
	sta screenbase+23*80,x
	lda screenbase+21*80,x
	sta screenbase+22*80,x
	lda screenbase+20*80,x
	sta screenbase+21*80,x
	lda screenbase+19*80,x
	sta screenbase+20*80,x
	lda screenbase+18*80,x
	sta screenbase+19*80,x
	lda screenbase+17*80,x
	sta screenbase+18*80,x
	lda screenbase+16*80,x
	sta screenbase+17*80,x
	lda screenbase+15*80,x
	sta screenbase+16*80,x
	lda screenbase+14*80,x
	sta screenbase+15*80,x
	lda screenbase+13*80,x
	sta screenbase+14*80,x
	lda screenbase+12*80,x
	sta screenbase+13*80,x
	lda screenbase+11*80,x
	sta screenbase+12*80,x
	lda screenbase+10*80,x
	sta screenbase+11*80,x
	lda screenbase+9*80,x
	sta screenbase+10*80,x
	lda screenbase+8*80,x
	sta screenbase+9*80,x
	lda screenbase+7*80,x
	sta screenbase+8*80,x
	lda screenbase+6*80,x
	sta screenbase+7*80,x
	lda screenbase+5*80,x
	sta screenbase+6*80,x
	lda screenbase+4*80,x
	sta screenbase+5*80,x

	dex
	bne .scdnloop
	rts
 endif

 endif	; ifconst C128


;---------------------------------------
; clear directory list

 ifconst window
clrlist	jsr setlistwin
	lda #147
	jsr bsout
	jsr setfullwin
	lda #" "
	jsr printcsr
 else

clrlist	subroutine
	clc	; jump to row 4 col 0
	ldx #4
	ldy #0
	jsr plot

	ldy #20

.clrloop	ldx #27
	lda #" "
.clrline	jsr bsout
	dex
	bpl .clrline

	lda #13	; next line
	jsr bsout

	dey
	bne .clrloop

	rts
 endif


;---------------------------------------
; load directory

getdir	jsr clrtables
	lda dir
	beq getssdir
	jmp getddir


; load directory from ss
getssdir	subroutine
	lda #1
	ldx #<dollar
	ldy #>dollar
	jsr setnam
	jsr ssload

	bcs .loaderror
	jsr parsedir
.loaderror	rts


; load directory from drive
getddir	subroutine
 ifconst C128
	lda #0
	tax
	jsr setbnk
 endif
	lda #1
	ldx #<dollar
	ldy #>dollar
	jsr setnam

	lda #0
	ldx iecdev
	ldy #0
	jsr setlfs

	lda #0
	ldx #<buffer
	ldy #>buffer
	jsr load

	bcs .kloaderror
	jsr parsedir
.kloaderror	rts


; parse the loaded directory
parsedir	lda #<buffer
	sta ptr
	lda #>buffer
	sta ptr+1

	ldy #3	; skip title
skip1st	iny
	lda (ptr),y
	bne skip1st
	iny

	tya
	clc
	adc ptr
	sta ptr
	lda #0
	adc ptr+1
	sta ptr+1

	lda #0
	sta nfiles

nextline	ldy #1	; if msb of link to next
	lda (ptr),y	; line is 0 we're
	beq listdone	; done

	ldx nfiles

	iny	; line number to AX
	lda (ptr),y
	sta sizel,x
	iny
	lda (ptr),y
	sta sizem,x

lprint	iny
	lda (ptr),y	; grab next char
	beq addyptr	; 0 = EOL
	sta ntmp-4,y
	jmp lprint

addyptr	iny	; skip 0 byte
	tya	; add Y to ptr
	tax	; X = Y
	clc
	adc ptr
	sta ptr
	lda ptr+1
	adc #0
	sta ptr+1

	txa	; len = Y-5
	sbc #5
	ldx nfiles
	jsr addname
	bcs nextline	; carry set if the entry sucks

	inc nfiles
	lda nfiles
	cmp #maxentries
	beq listdone
	jmp nextline
listdone
	rts


;---------------------------------------
; X = entry, A = len
addname	stx anentry
	txa	; get pointer to
	jsr name2tmp	; name in tmp

	ldx #0	; find \"
	lda ntmp
	cmp #34
	beq foundfnt
findfnut	lda ntmp,x
	cmp #34
	beq foundfnt
	cmp #$20
	bne nofname
	inx
	bne findfnut

nofname	sec
	rts

foundfnt	inx	; skip it

	ldy #0	; copy name to (tmp)
copyname	lda ntmp,x
	cmp #34	; until we hit \"
	beq cpnamedn
	sta (tmp),y
	inx
	iny
	bne copyname
cpnamedn
	lda #0	; pad name with 0
padname0	cpy #16	; max len
	beq pad0done
	sta (tmp),y
	iny
	bne padname0
pad0done

findtyp	inx
	beq notype	; no type?
	lda ntmp,x	; eat spaces until we find
	and #$7f	; the file type
	cmp #$20
	bne foundtyp
	beq findtyp

notype	sec	; no type
	rts
foundtyp
	cmp #"D"	; Dir or Del?
	bne notdord

	tay
	lda ntmp+1,x
	cmp #"E"	; dEl?
	bne notdel

	sec	; del, skip file
	rts
notdel
	tya
notdord
	ldx anentry	; store type in table
	sta type,x

	clc	; k, done
	rts

anentry	dc.b 0


;---------------------------------------
; LOAD

ssload	lda sborder
	pha
	sei

	jsr ser_init

	jsr ser_flush

	lda #"<"
	jsr sendname

	jsr ser_rx; receive one byte
	cmp #"?"
	beq notfound
	cmp #"!"
	beq getlen
	cmp #"."
	bne ldevnp
	pla
	sta sborder
	cli
	clc
	rts

ldevnp	pla
	sta sborder
	lda #5	; device not present
	cli
	sec
	rts

notfound	pla
	sta sborder
	lda #4	; file not found
	cli
	sec
	rts

getlen
	jsr ser_rx	; get length
	eor #$ff	; store 2's complement
	sta eal	; lsb
	jsr ser_rx
	eor #$ff
	sta eah	; msb
	lda #3	; +1 for 2's complement
	clc	; +2 to compensate for
	adc eal; load address
	sta eal
	lda #0
	adc eah
	sta eah

	jsr ser_rx	; get start addr
	sta tmp	; lsb
	jsr ser_rx
	sta tmp+1	; msb

	lda #<buffer	; load to buffer
	sta sal
	lda #>buffer
	sta sah

	ldy #0
recloop	jsr ser_rx
	sta (sal),y
	iny
	bne ynotwrap
	inc sah
ynotwrap
	inc eal
	bne notmsb
	inc eah
	beq recdone
notmsb	jmp recloop

recdone	tya	; add y to lsb
	clc
	adc sal
	sta eal	; copy to end address
	tax
	lda #0
	adc sah
	sta eah
	tay

	pla
	sta sborder
	cli
	clc	; all is well
	rts


;---------------------------------------
; SAVE

save	sei	; disable irqs
	lda sborder	; store color so we
	pha	; can flash

	jsr ser_flush

	lda #">"
	jsr sendname

	jsr ser_rx	; receive one byte
	cmp #"?"
	beq saveerr
	cmp #"!"
	beq sendlen

saveerr	pla
	sta sborder
	lda #3
	sta status
	lda #7	; not write file :)
	cli
	sec
	rts

sendlen	lda #<buffer	; fixed address
	sta stal
	lda #>buffer
	sta stah

	lda eal
	sec
	sbc stal
	jsr ser_tx	; send length lsb
	lda eah
	sbc stah
	jsr ser_tx	; send length msb

	jsr copystart	; start addr to current addr

	lda tmp	; send start addr
	jsr ser_tx
	lda tmp+1
	jsr ser_tx

	ldy #0

sendnext	jsr checkend	; check if end has been reached
	bcs saved

	lda (sal),y	; send a byte
	jsr ser_tx

	inc sal	; inc current addr
	bne notad
	inc sah
notad
	bne sendnext

saved	pla
	sta sborder
	lda #0
	sta status
	cli
	clc
	rts


checkend	sec	; checks on reaching the end address
	lda sal
	sbc eal
	lda sah
	sbc eah
	rts


copystart	lda stah	; copy start addr to current addr
	sta sah
	lda stal
	sta sal
	rts


;---------------------------------------
; IEC load routine

iecload	lda #1	; 1,d,0
	ldx iecdev
	ldy #0
	jsr setlfs

 ifconst C128
	lda #0
	ldx #0
	jsr setbnk
 endif
	jsr open	; open the file
	bcs ilfailed

	ldx #1	; set input channel
	jsr chkin
	bcs ilclose

	lda #<buffer	; write to buffer
	sta ptr
	lda #>buffer
	sta ptr+1

	lda sborder
	pha

	; ss load compatibility
	jsr basin	; get loadaddr lsb
	bcs ildone
	bit status
	bvs ildone
	sta tmp
	jsr basin	; get loadaddr msb
	bcs ildone
	bit status
	bvs ildone
	sta tmp+1	; store in tmp

	ldy #0
illoop	jsr basin
	bcs ildone
	sta (ptr),y
	sta sborder
	bit status
	bvs ildone
	inc ptr
	bne illoop
	inc ptr+1
	bne illoop

	brk	; farkle, shouldn't happen

ildone	pla
	sta sborder
	lda ptr	; save end address+1
	clc
	adc #1
	sta eal
	lda ptr+1
	adc #0
	sta eah
	;clc

ilclose	php
	lda #1
	jsr close
	jsr clrch
	plp

ilfailed	rts


iecname	dc.b "0123456789ABCDEF,X,X"


;---------------------------------------
; IEC save routine

iecsave	lda #1	; 1,d,1
	ldx iecdev
	ldy #1
	jsr setlfs

 ifconst C128
	lda #0
	ldx #0
	jsr setbnk
 endif
	jsr open	; open the file
	bcs isfailed

	ldx #1	; set input channel
	jsr ckout
	bcs isclose

	lda #<buffer	; fixed address
	sta stal
	lda #>buffer
	sta stah

	jsr copystart	; start addr to sal

	lda sborder
	pha

	lda tmp	; save start addr
	jsr bsout
	bcs isclose
	lda tmp+1
	jsr bsout
	bcs isclose

	ldy #0

savenext	jsr checkend	; check if end has been reached
	bcs issaved

	lda (sal),y	; send a byte
	jsr bsout
	bcs isdone

	inc sal	; inc current addr
	bne isnotad
	inc sah
isnotad
	bne savenext

	brk	; shouldn't happen

issaved	clc

isdone	pla
	sta sborder

isclose	php
	lda #1
	jsr close
	jsr clrch
	plp

isfailed	rts


;---------------------------------------
; Misc subroutines

 ifconst window
setlistwin	lda #4	; set window to 1,4 to 23,27
	ldx #1
	clc
	jsr window
	lda #23
	ldx #27
	sec
	jmp window

setfullwin	lda #0	; set window to 0,0 to 79,24
	ldx #0
	clc
	jsr window
	lda #24
	ldx linelen
	dex
	sec
	jmp window
 endif

 ifconst C128
; store A to VDC reg X
stv	subroutine
	stx $d600
.waitwrite	bit $d600
	bpl .waitwrite
	sta $d601
	rts

; read VDC reg X and return in A
ldv	subroutine
	stx $d600
.waitread	bit $d600
	bpl .waitread
	lda $d601
	rts
 endif


; print file type
prntype	ldx #0
testtype	cmp types,x
	beq gottype
	inx
	cpx #5
	bne testtype
gottype	txa
	asl
	asl
	tax
	ldy #3
ptyploop	lda typestr,x
	jsr bsout
	inx
	dey
	bne ptyploop
	rts

types	dc.b "P","S","U","R","D"
typestr	dc.b "PRG "
	dc.b "SEQ "
	dc.b "USR "
	dc.b "REL "
	dc.b "DIR "
typeunkn	dc.b "??? "


; send name to ss
sendname	;lda #"<"	; first byte to send
	jsr ser_tx	; in A
	ldy #0
prnname	lda (fnadr),y
	jsr ser_tx
	iny
	cpy fnlen
	bne prnname
	lda #13	; cr
	jsr ser_tx
	lda #10	; lf
	jsr ser_tx
	rts


name2tmp	pha
	lda #0	; tmp = X*16...
	sta tmp+1
	pla

	asl
	rol tmp+1
	asl
	rol tmp+1
	asl
	rol tmp+1
	asl
	rol tmp+1
	sta tmp

	;clc
	lda #<name	; ...+name
	adc tmp
	sta tmp
	lda #>name
	adc tmp+1
	sta tmp+1
	rts


; print string in AX -> string,0
pstring	sta tmp
	stx tmp+1
	tya
	pha
	ldy #0
pstrchar	lda (tmp),y
	beq pstrdone
	jsr bsout
	iny
	bne pstrchar
pstrdone
	pla
	tay
	rts


; this should probably be rewritten to use kernal routines
waitkey	lda ndx	; n of keys in buffer
	beq waitkey
	jmp getin


; set cursor to column in A
setcol	pha
	sec
	jsr plot	; get crsr pos
	pla
	tay	; new col = A
	clc
	jsr plot	; set crsr pos
	rts


; make an ascii -> screen code table
mkasctab	ldx #0
mkasclop	txa

	pha
	and #$1f
	sta asclsb
	pla
	lsr
	lsr
	lsr
	lsr
	lsr
	tay
	lda acvtab,y
	asl
	asl
	asl
	asl
	asl
	ora asclsb
	sta scrtab,x

	inx
	bne mkasclop

	lda #94	; pi is the exception
	ldx #$ff
	sta scrtab,x

	rts

asclsb	dc.b 0
acvtab	dc.b 4	; 00-1f -> 80-9f
	dc.b 1	; 20-3f -> 20-3f
	dc.b 0	; 40-5f -> 00-1f
	dc.b 2	; 60-7f -> 40-5f
	dc.b 6	; 80-9f -> c0-df
	dc.b 3	; a0-bf -> 60-7f
	dc.b 2	; same as 60-7f
	dc.b 3	; same as a0-bf


;---------------------------------------
; utility library

	include "util.i"


;---------------------------------------
; Serial port code

	include "ss232.i"


;---------------------------------------
; Buffers
;
; the work buffers

maxentries	= 144	; max number of files

scrtab
ntmp	= scrtab+256
sizel	= ntmp+256
sizem	= sizel+maxentries
type	= sizem+maxentries
sel	= type+maxentries
name	= sel+maxentries
buffer	= name+(16*maxentries)	; rest of mem
