;---------------------------------------
; Serial Slave disk image reader/writer
;
; History
;  0.1  copied sscopy source


	processor 6502

	include "kernal.i"
	include "platform.i"
	include "ssconst.i"
	include "ascii.i"


dev	= 6
maxlines	= 20





;---------------------------------------
; INIT

init	subroutine
	tsx
	stx savestack
 ifconst C64
	lda #$26	; kernal, I/O
	sta $01
 endif

 ifconst C128
	lda #%00001110	; kernal, I/O, ram
	sta $ff00
 endif

	jsr scrorg	; get screen geometry
 ifconst C128
	tax	; C128 returns screen width -1 in A
	inx
 endif
	stx linelen

 ifconst C128
	cpx #80	; 80 col mode?
	bne .not80
		; set VDC colours
	ldx #$1a	; light grey background
	lda #14
	jsr stv

	jmp .skip40
.not80
 endif

; set VIC/TED colours
 ifnconst PET
	lda #bkgcol	; medium grey screen
	sta sbkg
	lda #0	; black border
	sta sborder
 endif

.skip40

 ifconst C128
	jsr setfullwin	; in case we started with a window defined
 endif

	lda fa	; default device
	cmp #dev
	bne .notss
	lda #8
.notss
	sta iecdev
	lda #0	; copy from SS
	sta dir

	jsr primm
	dc.b a_black,"OPENWCHANNEL ",0
	jsr openwchannel
	bcs .error
	jsr primm
	dc.b "OK",13,0

	ldx #0
.loop	txa
	sta buffer,x
	inx
	bne .loop

	sta sector
	lda #1
	sta track

	jsr primm
	dc.b "WRITETS ",0
	jsr writets
	bcs .error
	jsr primm
	dc.b "OK",13,0

	jsr primm
	dc.b "CLOSEWCHANNEL ",0
	jsr closewchannel
	bcs .error
	jsr primm
	dc.b "OK",13,0

.done	lda #$27
	sta $01
	rts

.error	jsr primm
	dc.b "FAILED",13,0
	jsr closewchannel
	jmp .done

	lda #a_clr
	jsr bsout
	jsr ptitle
	jsr getdir
	jsr printlist


;---------------------------------------
; MAIN

main	jsr waitkey

	cmp #"D"
	bne notdev
	jsr nextdev
	jmp main
notdev
	cmp #17	; crsr down
	bne notdown
	jsr down
	jmp main
notdown
	cmp #145	; crsr up
	bne notup
	jsr up
	jmp main
notup
	;cmp #95	; <-
	;bne notswap
	;jsr swap
	;jmp main
notswap
	cmp #" "
	bne notselct
	jsr select
	jmp main
notselct
	cmp #13	; enter
	bne notstart
	jsr start
	jmp main

notstart
	cmp #"Q"
	bne notquit
quit	lda #a_clr
	jsr bsout

 ifconst C64
	lda #$27
	sta $01
 endif

 ifconst C128
	lda #%00000000
	sta $ff00
 endif

	rts
notquit
	cmp #20
	bne notparen
	jsr cdparen
	jmp main

notparen
	cmp #19
	bne nothome
	jsr gohome
	jmp main

nothome
	cmp #"R"
	bne notreread
	jsr reread
	jmp main

notreread
 ifnconst PET
	inc sborder
	ldx rastline
	dex
waitvblx	cpx rastline
	bne waitvblx
	dec sborder
 endif
	jmp main

	rts


;---------------------------------------
; BSS

dir	dc.b 0
iecdev	dc.b 0
savey	dc.b 0
savex	dc.b 0
nfiles	dc.b 0
curline	dc.b 0
offset	dc.b 0
linelen	dc.b 0
numbl	dc.b 0
numbh	dc.b 0
savestack	dc.b 0
track	dc.b 0
sector	dc.b 0
curblock	dc.w 0


;---------------------------------------
; DATA

dollar	dc.b "$"

titlescr	; clr, lc, black
	dc.b a_home, 14, 144
	; rvs on, ..., rvs off
	dc.b a_revon, "sERIAL sLAVE DISK IMAGE UTIL V0.1   2002", a_revoff
	dc.b 19, 17
	dc.b 5, "CRSR", 151, " AND ", 5, "SPACE", 151, " SELECTS, ", 5, "rETURN", 151, " STARTS", 13
	dc.b 5, 95, 151, " TO CHANGE TO/FROM ss, ", 5, "del", 151, " DOES CD ..  "
	dc.b 0

fromss	; home, down, white
	dc.b a_home,17,17,17
	dc.b "cURRENT DEST ", 5, "d", 151, "RIVE:     "
	; left, left
	dc.b 157,157,0

toss	; home, down
	dc.b a_home,17,17,17
	dc.b "cURRENT SOURCE ", 5, "d", 151, "RIVE:   "
	; left, left
	dc.b 157,157,0


;---------------------------------------
; Subs


;---------------------------------------
; clear tables

clrtables	subroutine
	lda #0
	sta nfiles
	sta curline
	sta offset
	tax
.loop	sta ntmp,x
	sta sizel,x
	sta sizem,x
	sta type,x
	inx
	cpx #maxentries
	bne .loop
	rts


;---------------------------------------
; print title

ptitle	ldx #0
	lda #<titlescr
	ldx #>titlescr
	jsr pstring
	lda #<fromss
	ldx #>fromss
	ldy dir
	beq pttoss
	lda #<toss
	ldx #>toss
pttoss
	jsr pstring
	ldx iecdev
	lda #0
	jsr print16
	lda #13
	jsr bsout
	rts


;---------------------------------------
; change to the next iec dev

nextdev	subroutine
	ldx iecdev
	inx
	txa
	and #7
	ora #8
	sta iecdev
	jsr ptitle
	lda dir	; no need to reload
	beq .done
	jsr getdir	; get iec dir
.done	jsr printlist
	rts


;---------------------------------------

swap	lda dir
	eor #1
	sta dir
	jsr ptitle
	;jsr getdir
	jsr printlist
	rts


;---------------------------------------
; reread directory

reread	lda #" "
	jsr printcsr
	jsr getdir
	jsr printlist
	rts


;---------------------------------------
; move cursor down

down	subroutine
	lda nfiles	; can't move if there aren't any files
	beq .done
	lda curline	; cl+of = nfiles?
	sec
	adc offset
	cmp nfiles
	bne .notdmax
	rts
.notdmax
	lda curline
	cmp #maxlines-1
	bne .notdbotm
	inc offset
	jsr scrollup
	lda #19
	jsr printline
	rts
.notdbotm
	lda #" "
	jsr printcsr
	inc curline
	lda #">"
	jsr printcsr
.done	rts


;---------------------------------------
; move cursor up

up	subroutine
	lda nfiles	; can't move if there aren't any files
	beq .done
	lda curline
	bne .notltop
	lda offset
	beq .noupoffs
	dec offset
	jsr scrolldown
	lda #0
	jsr printline
.noupoffs
	rts
.notltop
	lda #" "
	jsr printcsr
	dec curline
	lda #">"
	jsr printcsr
.done	rts


;---------------------------------------
; move cursor home

gohome	subroutine
	lda nfiles	; can't move if there aren't any files
	bne .dohome
	rts
.dohome	lda #0
	sta curline
	sta offset
	jmp printlist


;---------------------------------------
; mark an entry for copying

select	subroutine
	lda nfiles	; can't select if there aren't any files
	beq .done
	lda curline
	clc
	adc offset
	tax
	lda type,x
	cmp #"D"
	beq changdir
.done	rts

; change current directory
changdir	subroutine
	lda dir	; only cd on ss
	bne chgddone
	txa	; save line nr
	jsr xsetnam
docd	jsr ssload
	jsr ptitle
	jsr getdir
	jsr printlist
chgddone	rts


;---------------------------------------
; cd ..

cdparen	lda dir
	bne chgddone
	lda #2
	ldx #<dotdot
	ldy #>dotdot
	jsr setnam
	jmp docd

dotdot	dc.b ".."


;---------------------------------------
; start the copying

start	subroutine
	lda nfiles
	bne .checktype
	rts
.checktype	lda curline
	clc
	adc offset
	tax
	lda type,x
	cmp #"D"
	beq .checksize
.nope	rts
.checksize	lda sizel,x	; check that it's 689 blocks
	cmp #177
	bne .nope
	lda sizem,x
	cmp #2
	bne .nope

	jsr clrlist

	ldx #4	; jump to 0,4
	ldy #0
	clc
	jsr plot

	lda curline
	clc
	adc offset
	pha
	jsr xsetnam

	jsr primm
	dc.b "OPENING "
	dc.b 0

	pla
	pha
	jsr printnam

	lda dir
	beq cpfromss
	jmp cptoss
cpfromss
	jsr ssbload	; buffer file for transfer
	bcc .nocperror
	jmp cperror
.nocperror
	; should check size of the buffered file

	jsr primm
	dc.b " OK",13,"OPENING DEVICE ",0
	lda #0
	ldx iecdev
	jsr print16

	jsr openwchannel
	bcc .noowerror
	jmp cperror
.noowerror
	jsr primm
	dc.b " OK",13,0

	lda #1	; start at track 1 and block 0
	sta track
	lda #0
	sta curblock
	sta curblock+1

.nexttrack	jsr primm	; print the track number
	dc.b a_revoff,13,a_black,"TRACK ",0
	lda #0
	ldx track
	jsr print16
	jsr primm
	dc.b " ",a_revon,0

	lda #0
	sta sector

.nextsector	jsr ssbread	; read curblock to buffer
	lda #a_black	; print black
	bcc .norerror
	lda #a_red	; or red
.norerror	php
	jsr bsout
	lda #"R"	; R
	jsr bsout
	plp
	php
	bcs .norleft
	lda #a_left
	jsr bsout
.norleft
	plp
	bcs .skipwrite

	jsr writets	; write block to current track, sector
	lda #a_black	; print black
	bcc .nowerror
	lda #a_red	; or red
.nowerror	jsr bsout
	lda #"W"	; W
	jsr bsout
.skipwrite
	inc curblock
	bne .skip
	inc curblock+1
.skip
	inc sector
	lda sector
	ldx track
	cmp g1541tab,x
	bne .nextsector

	inc track
	lda track
	cmp g1541tab+1
	bne .nexttrack

	jsr closewchannel

	jmp cpdone

cptoss	jmp cperror

cperror	jsr primm
	dc.b " FAILED"
	dc.b 0

cpdone	jsr primm
	dc.b a_revoff,13,13,a_black
	dc.b "PRESS SPACE TO CONTINUE OR Q TO QUIT"
	dc.b 0

.waitspcorq	jsr waitkey	; wait for space or q
	cmp #" "
	beq .gotspace
	cmp #"Q"
	bne .waitspcorq

	ldx savestack	; chew return address
	txs
	jmp quit

.gotspace	jsr ptitle	; redraw screen
	jsr printlist
	rts


;---------------------------------------
; print filename

printnam	pha
	lda #34	; \"
	jsr bsout

	pla	; get pointer to
	jsr name2tmp	; name in tmp

	ldy #0	; print name
pname	lda (tmp),y
	beq pnamedn
	jsr bsout
	iny
	cpy #16
	bne pname
pnamedn
	lda #34
	jsr bsout
	rts


;---------------------------------------
; call setnam for the file in A

xsetnam	pha
 ifconst C128
	lda #0
	tax
	jsr setbnk
	pla
	pha
 endif
	jsr name2tmp
	ldy #0
xsgetlen	lda (tmp),y
	beq xsgotlen
	iny
	cpy #16
	bne xsgetlen
xsgotlen	tya
	ldx tmp
	ldy tmp+1
	jsr setnam
	pla
	rts


;---------------------------------------
; print the directory listing

printlist	subroutine
	lda nfiles
	bne nfnot0
	rts
nfnot0
	jsr clrlist

	lda #0
	sta .line
.drawloop	lda .line
	jsr printline
	inc .line
	lda .line
	cmp nfiles
	beq .done
	cmp #20
	bne .drawloop
.done
	lda #">"
	jsr printcsr
	rts

.line	dc.b 0


;---------------------------------------
; print line A in dir list

printline	subroutine
	pha

	clc
	adc offset
	sta ol	; keep track of line+offset

	pla
	clc	; move cursor to line + 4
	adc #4
	tax
	ldy #2
	clc
	jsr plot	; set cursor

	lda #144	; black
	jsr bsout

	ldx #23	; print 23 spaces
	lda #" "
clrline	jsr bsout
	dex
	bne clrline

	lda #1	; back to column 1
	jsr setcol

	ldx ol
	lda sizem,x
	pha
	lda sizel,x
	tax
	pla
	jsr print16	; print size

	lda #6	; jump to col 6
	jsr setcol

	lda ol	; print name
	jsr printnam

	lda #25	; jump to col 25
	jsr setcol

	ldx ol
	lda type,x	; print type
	jsr prntype

	rts


;---------------------------------------
; print A at cursor

printcsr	pha
	lda curline
	clc
	adc #4
	tax	; cursor to 0,4+curline
	ldy #0
	clc
	jsr plot	; set cursor

	lda #144	; black
	jsr bsout

	pla	; print character
	jsr bsout
	rts


;---------------------------------------
; scroll list up

 ifconst window
scrollup	jsr setlistwin
	jsr primm	; Esc-V to scroll up
	dc.b 27,"V",0
	jmp setfullwin
 else

scrollup	subroutine
 ifconst PET
	lda linelen	; handle 80 column mode scrolling
	cmp #80	; on pet
	beq .scup80col
 endif
	ldx #27
.scuploop	lda screenbase+5*40,x
	sta screenbase+4*40,x
	lda screenbase+6*40,x
	sta screenbase+5*40,x
	lda screenbase+7*40,x
	sta screenbase+6*40,x
	lda screenbase+8*40,x
	sta screenbase+7*40,x
	lda screenbase+9*40,x
	sta screenbase+8*40,x
	lda screenbase+10*40,x
	sta screenbase+9*40,x
	lda screenbase+11*40,x
	sta screenbase+10*40,x
	lda screenbase+12*40,x
	sta screenbase+11*40,x
	lda screenbase+13*40,x
	sta screenbase+12*40,x
	lda screenbase+14*40,x
	sta screenbase+13*40,x
	lda screenbase+15*40,x
	sta screenbase+14*40,x
	lda screenbase+16*40,x
	sta screenbase+15*40,x
	lda screenbase+17*40,x
	sta screenbase+16*40,x
	lda screenbase+18*40,x
	sta screenbase+17*40,x
	lda screenbase+19*40,x
	sta screenbase+18*40,x
	lda screenbase+20*40,x
	sta screenbase+19*40,x
	lda screenbase+21*40,x
	sta screenbase+20*40,x
	lda screenbase+22*40,x
	sta screenbase+21*40,x
	lda screenbase+23*40,x
	sta screenbase+22*40,x
	dex
	bne .scuploop
	rts

 ifconst PET
.scup80col	ldx #27
.scup80loop	lda screenbase+5*80,x	; fixme for 80-col mode
	sta screenbase+4*80,x
	lda screenbase+6*80,x
	sta screenbase+5*80,x
	lda screenbase+7*80,x
	sta screenbase+6*80,x
	lda screenbase+8*80,x
	sta screenbase+7*80,x
	lda screenbase+9*80,x
	sta screenbase+8*80,x
	lda screenbase+10*80,x
	sta screenbase+9*80,x
	lda screenbase+11*80,x
	sta screenbase+10*80,x
	lda screenbase+12*80,x
	sta screenbase+11*80,x
	lda screenbase+13*80,x
	sta screenbase+12*80,x
	lda screenbase+14*80,x
	sta screenbase+13*80,x
	lda screenbase+15*80,x
	sta screenbase+14*80,x
	lda screenbase+16*80,x
	sta screenbase+15*80,x
	lda screenbase+17*80,x
	sta screenbase+16*80,x
	lda screenbase+18*80,x
	sta screenbase+17*80,x
	lda screenbase+19*80,x
	sta screenbase+18*80,x
	lda screenbase+20*80,x
	sta screenbase+19*80,x
	lda screenbase+21*80,x
	sta screenbase+20*80,x
	lda screenbase+22*80,x
	sta screenbase+21*80,x
	lda screenbase+23*80,x
	sta screenbase+22*80,x
	dex
	bne .scup80loop
	rts
 endif

 endif	; ifconst C128


;---------------------------------------
; scroll list down

 ifconst window
scrolldown	jsr setlistwin
	jsr primm	; Esc-W to scroll down
	dc.b 27,"W",0
	jmp setfullwin
 else

scrolldown	subroutine
 ifconst PET
	lda linelen	; handle 80 column mode scrolling
	cmp #80	; on pet
	beq .scdn80col
 endif
	ldx #27
.scdnloop
	lda screenbase+22*40,x
	sta screenbase+23*40,x
	lda screenbase+21*40,x
	sta screenbase+22*40,x
	lda screenbase+20*40,x
	sta screenbase+21*40,x
	lda screenbase+19*40,x
	sta screenbase+20*40,x
	lda screenbase+18*40,x
	sta screenbase+19*40,x
	lda screenbase+17*40,x
	sta screenbase+18*40,x
	lda screenbase+16*40,x
	sta screenbase+17*40,x
	lda screenbase+15*40,x
	sta screenbase+16*40,x
	lda screenbase+14*40,x
	sta screenbase+15*40,x
	lda screenbase+13*40,x
	sta screenbase+14*40,x
	lda screenbase+12*40,x
	sta screenbase+13*40,x
	lda screenbase+11*40,x
	sta screenbase+12*40,x
	lda screenbase+10*40,x
	sta screenbase+11*40,x
	lda screenbase+9*40,x
	sta screenbase+10*40,x
	lda screenbase+8*40,x
	sta screenbase+9*40,x
	lda screenbase+7*40,x
	sta screenbase+8*40,x
	lda screenbase+6*40,x
	sta screenbase+7*40,x
	lda screenbase+5*40,x
	sta screenbase+6*40,x
	lda screenbase+4*40,x
	sta screenbase+5*40,x

	dex
	bne .scdnloop
	rts

 ifconst PET
.scdn80col	ldx #27
.scdn80loop
	lda screenbase+22*80,x
	sta screenbase+23*80,x
	lda screenbase+21*80,x
	sta screenbase+22*80,x
	lda screenbase+20*80,x
	sta screenbase+21*80,x
	lda screenbase+19*80,x
	sta screenbase+20*80,x
	lda screenbase+18*80,x
	sta screenbase+19*80,x
	lda screenbase+17*80,x
	sta screenbase+18*80,x
	lda screenbase+16*80,x
	sta screenbase+17*80,x
	lda screenbase+15*80,x
	sta screenbase+16*80,x
	lda screenbase+14*80,x
	sta screenbase+15*80,x
	lda screenbase+13*80,x
	sta screenbase+14*80,x
	lda screenbase+12*80,x
	sta screenbase+13*80,x
	lda screenbase+11*80,x
	sta screenbase+12*80,x
	lda screenbase+10*80,x
	sta screenbase+11*80,x
	lda screenbase+9*80,x
	sta screenbase+10*80,x
	lda screenbase+8*80,x
	sta screenbase+9*80,x
	lda screenbase+7*80,x
	sta screenbase+8*80,x
	lda screenbase+6*80,x
	sta screenbase+7*80,x
	lda screenbase+5*80,x
	sta screenbase+6*80,x
	lda screenbase+4*80,x
	sta screenbase+5*80,x

	dex
	bne .scdnloop
	rts
 endif

 endif	; ifconst C128


;---------------------------------------
; clear directory list

 ifconst window
clrlist	jsr setlistwin
	lda #a_clr
	jsr bsout
	jsr setfullwin
	lda #" "
	jsr printcsr
 else

clrlist	subroutine
	clc	; jump to row 4 col 0
	ldx #4
	ldy #0
	jsr plot

	ldy #20

.clrloop	ldx #27
	lda #" "
.clrline	jsr bsout
	dex
	bpl .clrline

	lda #13	; next line
	jsr bsout

	dey
	bne .clrloop

	rts
 endif


;---------------------------------------
; load directory

getdir	jsr clrtables
	lda dir
	beq getssdir
	jmp getddir


; load directory from ss
getssdir	subroutine
	lda #1
	ldx #<dollar
	ldy #>dollar
	jsr setnam
	jsr ssload

	bcs .loaderror
	jsr parsedir
.loaderror	rts


; load directory from drive
getddir	subroutine
 ifconst C128
	lda #0
	tax
	jsr setbnk
 endif
	lda #1
	ldx #<dollar
	ldy #>dollar
	jsr setnam

	lda #0
	ldx iecdev
	ldy #0
	jsr setlfs

	lda #0
	ldx #<buffer
	ldy #>buffer
	jsr load

	bcs .kloaderror
	jsr parsedir
.kloaderror	rts


; parse the loaded directory
parsedir	lda #<buffer
	sta ptr
	lda #>buffer
	sta ptr+1

	ldy #3	; skip title
skip1st	iny
	lda (ptr),y
	bne skip1st
	iny

	tya
	clc
	adc ptr
	sta ptr
	lda #0
	adc ptr+1
	sta ptr+1

	lda #0
	sta nfiles

nextline	ldy #1	; if msb of link to next
	lda (ptr),y	; line is 0 we're
	beq listdone	; done

	ldx nfiles

	iny	; line number to AX
	lda (ptr),y
	sta sizel,x
	iny
	lda (ptr),y
	sta sizem,x

lprint	iny
	lda (ptr),y	; grab next char
	beq addyptr	; 0 = EOL
	sta ntmp-4,y
	jmp lprint

addyptr	iny	; skip 0 byte
	tya	; add Y to ptr
	tax	; X = Y
	clc
	adc ptr
	sta ptr
	lda ptr+1
	adc #0
	sta ptr+1

	txa	; len = Y-5
	sbc #5
	ldx nfiles
	jsr addname
	bcs nextline	; carry set if the entry sucks

	inc nfiles
	lda nfiles
	cmp #maxentries
	beq listdone
	jmp nextline
listdone
	rts


;---------------------------------------
; X = entry, A = len
addname	stx anentry
	txa	; get pointer to
	jsr name2tmp	; name in tmp

	ldx #0	; find \"
	lda ntmp
	cmp #34
	beq foundfnt
findfnut	lda ntmp,x
	cmp #34
	beq foundfnt
	cmp #$20
	bne nofname
	inx
	bne findfnut

nofname	sec
	rts

foundfnt	inx	; skip it

	ldy #0	; copy name to (tmp)
copyname	lda ntmp,x
	cmp #34	; until we hit \"
	beq cpnamedn
	sta (tmp),y
	inx
	iny
	bne copyname
cpnamedn
	lda #0	; pad name with 0
padname0	cpy #16	; max len
	beq pad0done
	sta (tmp),y
	iny
	bne padname0
pad0done

findtyp	inx
	beq notype	; no type?
	lda ntmp,x	; eat spaces until we find
	and #$7f	; the file type
	cmp #$20
	bne foundtyp
	beq findtyp

notype	sec	; no type
	rts
foundtyp
	cmp #"D"	; Dir or Del?
	bne notdord

	tay
	lda ntmp+1,x
	cmp #"E"	; dEl?
	bne notdel

	sec	; del, skip file
	rts
notdel
	tya
notdord
	ldx anentry	; store type in table
	sta type,x

	clc	; k, done
	rts

anentry	dc.b 0


;---------------------------------------
; LOAD

ssload	lda sborder
	pha
	sei

	jsr ser_init

	lda #"<"
	jsr sendname

	jsr ser_rx; receive one byte
	cmp #"?"
	beq notfound
	cmp #"!"
	beq getlen
	cmp #"."
	bne ldevnp
	pla
	sta sborder
	cli
	clc
	rts

ldevnp	pla
	sta sborder
	lda #5	; device not present
	cli
	sec
	rts

notfound	pla
	sta sborder
	lda #4	; file not found
	cli
	sec
	rts

getlen
	jsr ser_rx	; get length
	eor #$ff	; store 2's complement
	sta eal	; lsb
	jsr ser_rx
	eor #$ff
	sta eah	; msb
	lda #3	; +1 for 2's complement
	clc	; +2 to compensate for
	adc eal; load address
	sta eal
	lda #0
	adc eah
	sta eah

	jsr ser_rx	; get start addr
	sta tmp	; lsb
	jsr ser_rx
	sta tmp+1	; msb

	lda #<buffer	; load to buffer
	sta sal
	lda #>buffer
	sta sah

	ldy #0
recloop	jsr ser_rx
	sta (sal),y
	iny
	bne ynotwrap
	inc sah
ynotwrap
	inc eal
	bne notmsb
	inc eah
	beq recdone
notmsb	jmp recloop

recdone	tya	; add y to lsb
	clc
	adc sal
	sta eal	; copy to end address
	tax
	lda #0
	adc sah
	sta eah
	tay

	pla
	sta sborder
	cli
	clc	; all is well
	rts


;---------------------------------------
; Block load

ssbload	subroutine
	lda sborder
	pha
	sei

	jsr ser_init

	lda #"B"
	jsr ser_tx
	lda #"L"
	jsr sendname

	jsr ser_rx
	cmp #"?"
	beq .notfound
	cmp #"!"
	beq .getlen
	cmp #"."
	bne .ldevnp
	pla
	sta sborder
	cli
	clc
	rts

.ldevnp	pla
	sta sborder
	lda #5	; device not present
	cli
	sec
	rts

.notfound	pla
	sta sborder
	lda #4	; file not found
	cli
	sec
	rts

.getlen	jsr ser_rx	; get and store block count
	sta numbl
	jsr ser_rx
	sta numbh

	pla	; ready to rock'n'roll
	sta sborder
	cli
	clc
	rts


;---------------------------------------
; read a block to buffer

ssbread	subroutine
	sei
	lda sborder
	pha

	jsr ser_init

	lda #"B"
	jsr ser_tx
	lda #"R"
	jsr ser_tx
	jsr sendcurblock

	jsr ser_rx
	cmp #"?"
	beq .notfound
	cmp #"!"
	beq .getlen
	cmp #"."
	bne .ldevnp
	pla
	sta sborder
	cli
	clc
	rts

.ldevnp	pla
	sta sborder
	lda #5	; device not present
	cli
	sec
	rts

.notfound	pla
	sta sborder
	lda #4	; file not found
	cli
	sec
	rts

.getlen	jsr ser_rx	; get and store block length
	sta blen
	ldx #0
	stx cksum

.nextbyte	jsr ser_rx
	sta buffer,x
	clc
	adc cksum
	sta cksum
	cpx blen
	beq .gotblock
	inx
	bne .nextbyte
.gotblock
	inx	; fill rest of buffer with 0s
	beq .got256
	lda #0
.clearend	sta buffer,x
	inx
	bne .clearend
.got256
	jsr ser_rx
	cmp cksum
	beq .ok

	pla
	sta sborder
	sec
	cli
	rts

.ok	pla
	sta sborder
	clc
	cli
	rts


cksum	dc.b 0
blen	dc.b 0


;---------------------------------------
; open write channel

openwchannel	subroutine
	lda #0	; open command channel
	tax
	tay
	jsr setnam

 ifconst C128
	lda #0
	tax
	jsr setbnk
 endif

	lda #15
	ldx iecdev
	ldy #15
	jsr setlfs

	jsr open
	bcs .done

	lda #1	; open data channel
	ldx #<.hash
	ldy #>.hash
	jsr setnam

	lda #8
	ldx iecdev
	ldy #8
	jsr setlfs

	jsr open
	bcc .done

	lda #15	; close the command channel
	jsr close
	sec

.done	rts

.hash	dc.b "#"


;---------------------------------------
; open write channel

closewchannel	subroutine
	lda #8
	jsr close
	lda #15
	jsr close
	rts


;---------------------------------------
; write current track, sector

writets	subroutine
	ldx #15	; select command channel
	jsr ckout
	bcs .done

	ldx #0	; send b-p
.sendbp0	lda .bp0,x
	jsr bsout
	inx
	cpx #9
	bne .sendbp0

	ldx #8	; select data channel
	jsr ckout
	bcs .done

	ldx #0
.senddata	lda buffer,x
	jsr bsout
	inx
	bne .senddata

	lda #"0"	; convert track to ascii
	sta .track
	lda track
.dect10	cmp #10
	bcc .donet
	sbc #10
	inc .track
	bne .dect10
.donet	clc
	adc #"0"
	sta .track+1

	lda #"0"	; convert sector to ascii
	sta .sector
	jmp .foo
	lda sector
.decs10	cmp #10
	bcc .dones
	sbc #10
	inc .sector
	bne .decs10
.dones	clc
	adc #"0"
	sta .sector+1
.foo
	ldx #15	; select command channel
	jsr ckout
	bcs .done

	ldx #0	; send U2 command
.sendu2	lda .blah,x
	sta $0400,x
	jsr bsout
	bcs .done
	inx
	cpx #.u2len
	bne .sendu2

	;clc

.done	php
	jsr clrch
	plp
	rts

.u2	dc.b "U2: 8 0 "
.track	dc.b "00 "
.sector	dc.b "00",13
.u2len	= * - .u2

.bp0	dc.b "B-P: 8 0",13	; fixme


;---------------------------------------
; Misc subroutines

 ifconst window
setlistwin	lda #4	; set window to 1,4 to 23,27
	ldx #1
	clc
	jsr window
	lda #23
	ldx #27
	sec
	jmp window

setfullwin	lda #0	; set window to 0,0 to 79,24
	ldx #0
	clc
	jsr window
	lda #24
	ldx linelen
	dex
	sec
	jmp window
 endif

 ifconst C128
; store A to VDC reg X
stv	subroutine
	stx $d600
.waitwrite	bit $d600
	bpl .waitwrite
	sta $d601
	rts

; read VDC reg X and return in A
ldv	subroutine
	stx $d600
.waitread	bit $d600
	bpl .waitread
	lda $d601
	rts
 endif


; print file type
prntype	ldx #0
testtype	cmp types,x
	beq gottype
	inx
	cpx #5
	bne testtype
gottype	txa
	asl
	asl
	tax
	ldy #3
ptyploop	lda typestr,x
	jsr bsout
	inx
	dey
	bne ptyploop
	rts

types	dc.b "P","S","U","R","D"
typestr	dc.b "PRG "
	dc.b "SEQ "
	dc.b "USR "
	dc.b "REL "
	dc.b "DIR "
typeunkn	dc.b "??? "


; send name to ss
sendname	jsr ser_tx	; first byte to send
	ldy #0	; in A
prnname	lda (fnadr),y
	jsr ser_tx
	iny
	cpy fnlen
	bne prnname
sendcrlf	lda #13	; cr
	jsr ser_tx
	lda #10	; lf
	jsr ser_tx
	rts


; send current block number to ss
sendcurblock	lda curblock
	jsr ser_tx
	lda curblock+1
	jsr ser_tx
	jmp sendcrlf


name2tmp	pha
	lda #0	; tmp = X*16...
	sta tmp+1
	pla

	asl
	rol tmp+1
	asl
	rol tmp+1
	asl
	rol tmp+1
	asl
	rol tmp+1
	sta tmp

	;clc
	lda #<name	; ...+name
	adc tmp
	sta tmp
	lda #>name
	adc tmp+1
	sta tmp+1
	rts


; print string in AX -> string,0
pstring	sta tmp
	stx tmp+1
	tya
	pha
	ldy #0
pstrchar	lda (tmp),y
	beq pstrdone
	jsr bsout
	iny
	bne pstrchar
pstrdone
	pla
	tay
	rts


; this should probably be rewritten to use kernal routines
waitkey	lda ndx	; n of keys in buffer
	beq waitkey
	jmp getin


; set cursor to column in A
setcol	pha
	sec
	jsr plot	; get crsr pos
	pla
	tay	; new col = A
	clc
	jsr plot	; set crsr pos
	rts


;---------------------------------------
; utility library

	include "util.i"


;---------------------------------------
; Serial port code

	include "ss232.i"


;---------------------------------------
; 1541 geometry

g1541tab	; 35 tracks
	dc.b 35
	; 21 sectors on track 1-17
	dc.b 21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21,21
	; 19 sectors on track 18-24
	dc.b 19,19,19,19,19,19,19
	; 18 sectors on track 25-30
	dc.b 18,18,18,18,18,18
	; 17 sectors on track 31-35
	dc.b 17,17,17,17,17


;---------------------------------------
; Buffers
;
; the work buffers

maxentries	= 144	; max number of files

ntmp
sizel	= ntmp+256
sizem	= sizel+maxentries
type	= sizem+maxentries
name	= type+maxentries
buffer	= name+(16*maxentries)	; rest of mem
