;---------------------------------------
; Serial Slave driver
;
; LOAD/SAVE patch that uses a SwiftLink
; or Turbo232 with a null modem cable
;
; History
;   0.1  LOAD
;   0.2  cleanup & bugfixes
;   0.3  SAVE
;   0.4  handle '.' result
;   0.5  support for 16550, and prepared
;        code for multiple targets
;        renamed source file
;   0.6  makes sure it's not already installed
;        before hijacking vectors

; gpz added 20020828: ser_flush calls


	include "ssconst.i"


	processor 6502


 ifconst C64

 ifnconst init
init	= $ce00
 endif
loadvect	= $0330
savevect	= $0332
sborder	= $d020

 else
 ifconst C128

 ifnconst init
init	= $1a00
 endif
loadvect	= $0330
savevect	= $0332
sborder	= $d020

 else
 ifconst PLUS4
	echo "don't know where to put Plus/4 driver"
 ifnconst init
init	= $xxxx
 endif
loadvect	= $xxxx
savevect	= $xxxx
sborder	= $ff19

 else
 ifconst PET
	echo "this driver won't work with the PET"
 ifnconst init
init	= $xxxx
 endif
loadvect	= $xxxx
savevect	= $xxxx
sborder	= $ffff

 endif
 endif
 endif
 endif


	org init

;---------------------------------------
; INIT

	jmp skip

dev	dc.b 6	; org + 4 = device number

skip
	; check if we're already installed
	lda loadvect
	cmp #<myload
	bne install
	lda loadvect+1
	cmp #>myload
	beq installed

install	; tap into load
	lda loadvect
	sta jumpload+1
	lda loadvect+1
	sta jumpload+2

	lda #<myload
	sta loadvect
	lda #>myload
	sta loadvect+1

	; tap into save
	lda savevect
	sta jumpsave+1
	lda savevect+1
	sta jumpsave+2

	lda #<mysave
	sta savevect
	lda #>mysave
	sta savevect+1

	lda #255	; flash border
wait1	cmp $d012
	bne wait1
	inc $d020
	lda #254
wait2	cmp $d012
	bne wait2
	dec $d020

	; now sit back and wait
installed	rts


;---------------------------------------
; LOAD

myload	pha
	lda $ba	; device address
	cmp dev
	beq doload
	pla
jumpload	jmp $ffff	; changed by init

doload	pla
	beq yepload

	lda #4	; verify unimplemented
	sec
	rts

yepload	sta $93	; load/verify flag

	sei	; disable irqs
	lda sborder	; store color so we
	pha	; can flash

	lda #0
	sta $90	; clear status

	ldy $b7	; length of file name
	bne fnnot0
	jmp $f710	; missing file name

fnnot0	jsr ser_init

	jsr $f5af	; searching for ...

	jsr ser_flush

	lda #"<"
	jsr sendname

	jsr ser_rx; receive one byte
	cmp #"?"
	beq notfound
	cmp #"!"
	beq getlen
	cmp #"."
	bne ldevnp
	pla
	sta sborder
	cli
	clc
	rts

ldevnp	pla
	sta sborder
	lda #5	; device not present
	cli
	sec
	rts

notfound	pla
	sta sborder
	lda #4	; file not found
	cli
	sec
	rts

getlen	jsr $f5d2	; print loading
	jsr ser_rx	; get length
	eor #$ff	; store 2's complement
	sta $ae	; lsb
	jsr ser_rx
	eor #$ff
	sta $af	; msb
	lda #3	; +1 for 2's complement
	clc	; +2 to compensate for
	adc $ae	; load address
	sta $ae
	lda #0
	adc $af
	sta $af

	jsr ser_rx	; get start addr
	sta $ac	; lsb
	jsr ser_rx
	sta $ad	; msb

	lda $b9
	bne notbasic
	lda $c3	; load to given addr
	sta $ac
	lda $c4
	sta $ad
notbasic
	ldy #0
recloop	jsr ser_rx
	sta ($ac),y
	iny
	bne ynotwrap
	inc $ad
ynotwrap
	inc $ae
	bne notmsb
	inc $af
	beq recdone
notmsb	jmp recloop

recdone	tya	; add y to lsb
	clc
	adc $ac
	sta $ae	; copy to end address
	tax
	lda #0
	adc $ad
	sta $af
	tay

	pla
	sta sborder
	cli
	clc	; all is well
	rts


;---------------------------------------
; SAVE

mysave	lda $ba	; device address
	cmp dev
	beq dosave
jumpsave	jmp $ffff	; changed by init

dosave	lda $b7
	bne not0name
	jmp $f710	; missing file name
not0name
	sei	; disable irqs
	lda sborder	; store color so we
	pha	; can flash

	jsr ser_flush

	lda #">"
	jsr sendname

	jsr ser_rx	; receive one byte
	cmp #"."
	bne scheckok
	pla
	sta sborder
	cli
	clc
	rts

scheckok	cmp #"!"
	beq sendlen

	pla
	sta sborder
	lda #3
	sta $90
	lda #7	; "not a write file" error :)
	cli
	sec
	rts

sendlen	jsr $f68f	; output saving

	sec
	lda $ae
	sbc $c1
	jsr ser_tx	; send length lsb
	lda $af
	sbc $c2
	jsr ser_tx	; send length msb

	jsr $fb8e	; start addr to $ac

	lda $ac	; send start addr
	jsr ser_tx
	lda $ad
	jsr ser_tx

	ldy #0

sendnext	jsr $fcd1	; check if end reachd
	bcs saved

	lda ($ac),y	; send a byte
	jsr ser_tx

	inc $ac	; inc current addr
	bne notad
	inc $ad
notad
	bne sendnext

saved	pla
	sta sborder
	lda #0
	sta $90
	cli
	clc
	rts


;---------------------------------------
; Misc subroutines

sendname	;lda #"<"	; first byte to send
	jsr ser_tx	; in A
	ldy #0
prnname	lda ($bb),y
	jsr ser_tx
	iny
	cpy $b7
	bne prnname
	lda #13	; cr
	jsr ser_tx
	lda #10	; lf
	jsr ser_tx
	rts


;---------------------------------------
; Serial port code

	include "ss232.i"
