/*

Serial port test application

gpz fixed 20020828: changed references from RS_BAUD... to SER_...

*/


#include <stdlib.h>
#include <stdio.h>
#include <rs232.h>

#ifdef __C64__
#include <c64.h>
#endif

#ifdef __C128__
#include <c128.h>
#endif

#include "ssdisk.h"
#include "sscmd.h"
#include "serial.h"
#include "dir.h"
#include "ssui.h"
#include "diskio.h"


static unsigned char *baudratestr[] = {
  "wrong","wrong","wrong","wrong","wrong","wrong","wrong"
  "2400",    /* SER_2400   0x06 */
  "4800",    /* SER_4800   0x07 */
  "9600",    /* SER_9600   0x08 */
  "19200",   /* SER_19200  0x09 */
  "38400",   /* SER_38400  0x0A */
  "57600",   /* SER_57600  0x0B */
  "115200",  /* SER_115200 0x0C */
  "230400"   /* SER_230400 0x0D */
};


unsigned char cksum[MAXBLOCKS];


int main (void)
{
  unsigned char i;
  unsigned char c;
  unsigned char verify = 0;
  unsigned char ssver[40];
  unsigned char msg[39];
  unsigned char interface = SER_TURBO232;
  unsigned char baudrate = SER_115200;
  unsigned char termprog = 0;

  resetscreen("\x12Serial Slave serial port tester\xa9\xa9\xa9\xa9\xa9\xa9\xa9\xa9x\x92\n");

  (void) showrequester("Welcome", "Welcome to the SS Test Utility", REQ_OK);

  if (configserial(&interface, &baudrate) != OK) {
    return(EXIT_SUCCESS);
  }

  if (ser_init(interface, baudrate) != OK) {
    (void) showrequester("Error", "Couldn't initialize RS-232 interface", REQ_OK);
    return(EXIT_FAILURE);
  }

  if (showrequester("Test", "Test with terminal program on PC?", REQ_YESNO) == 1) {
    termprog = 1;
    sprintf(msg, "Set to %s baud 8N1", baudratestr[baudrate]);
    (void) showrequester("Success", msg, REQ_OK);
    (void) showrequester("Sending", "Sending \"hello\" to PC", REQ_OK);
    ser_send("hello\n\r", 7);
    if (showrequester("Test", "Did the PC print hello?", REQ_YESNO) != 1) {
       (void) showrequester("Failure", "Well, it *should* work.", REQ_OK);
    }
    printf("\x13\nType something on the PC\n\n");
    for (i = 0; i < 10; ++i) {
      c = ser_rx();
      printf("Received 0x%02x, '%c'\n", c, c);
    }
    if (showrequester("Test", "Did that work?", REQ_YESNO) != 1) {
       (void) showrequester("Failure", "Well, it *should* work.", REQ_OK);
    }
  }

  if (showrequester("Test", "Test with SS server on PC?", REQ_YESNO) == 1) {
    if (termprog == 1) {
      (void) showrequester("Test", "Quit terminal program", REQ_OK);
    }
    sprintf(msg, "Start serslave at %s baud", baudratestr[baudrate]);
    (void) showrequester("Test", msg, REQ_OK);
    printf("\x13\nPinging server...\n");
    if (pingss(&ssver) == OK) {
      (void) showrequester("Success", ssver, REQ_OK);
    } else {
      (void) showrequester("Error", "No SS server!", REQ_OK);
      ser_exit();
      return(EXIT_FAILURE);
    }
  }

  (void) showrequester("Done", "All tests done.", REQ_OK);

  myputchar(147);

  return(EXIT_SUCCESS);
}
