/*

User interface

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <rs232.h>

#include "ssdisk.h"
#include "dir.h"
#include "ssui.h"
#include "serial.h"


/* buttons for various functions */
button btnok = {
  "  OK  ",
  13
};

button btncancel = {
  "Cancel",
  3
};

button btnyes = {
  " Yes  ",
  89
};

button btnno = {
  "  No  ",
  78
};


/*

Set screen colors, hide the cursor, clear the screen, print title

*/
void resetscreen(unsigned char *title) {
  /* set screen colors, hide the cursor, clear the screen */
  textcolor(COLOR_BLACK);
  bordercolor(COLOR_GRAY1);
  bgcolor(COLOR_GRAY3);
  cursor(0);
  clrscr();

  myputs(title);
}


/* various GUI elements for configserial */
option cs_t232 = {
  "1. Swiftlink or Turbo232",
  '1',
  0, 0
};

option cs_silver = {
  "2. Silversurfer",
  '2',
  0, 1
};

option cs_baud115200 = {
  "3. 115200",
  '3',
  0, 0
};

option cs_baud57600 = {
  "4.  57600",
  '4',
  0, 1
};

option cs_baud38400 = {
  "5.  38400",
  '5',
  0, 2
};

button *cs_butokcancel[] = {
  &btncancel,
  &btnok,
  NULL
};

/*

Configure the serial interface

*/
unsigned char configserial(unsigned char *interface, unsigned char *baudrate) {
  char c;
  unsigned char newif, newbaud;
  window *win;
  unsigned char redraw = 1;

  newif = *interface;
  newbaud = *baudrate;

  if ((win = openwindow("Serial port settings",
#ifdef __C64__
			5, 5, 30, 15,
#else
			5, 10, 30, 10,
#endif
			WIN_CLOSEBUTTON, cs_butokcancel)) == NULL) {
    return(ERROR);
  }

  while (1) {
    if (redraw) {

#ifdef __C64__
      gotoxy(6, 7);
      myputs("Select RS-232 interface\n\n");

      gotoxy(cs_t232.x + 6, cs_t232.y + 9);
      newif == SER_TURBO232 ? putchar(18) : putchar(146);
      myputs(cs_t232.text);
      putchar(146);

      gotoxy(cs_silver.x + 6, cs_silver.y + 9);
      newif == SER_SILVER ? putchar(18) : putchar(146);
      myputs(cs_silver.text);
      putchar(146);
#endif

      gotoxy(6, 12);
      myputs("Select baudrate");

      gotoxy(cs_baud115200.x + 6, cs_baud115200.y + 14);
      newbaud == SER_115200 ? putchar(18) : putchar(146);
      myputs(cs_baud115200.text);
      putchar(146);

      gotoxy(cs_baud57600.x + 6, cs_baud57600.y + 14);
      newbaud == SER_57600 ? putchar(18) : putchar(146);
      myputs(cs_baud57600.text);
      putchar(146);

      gotoxy(cs_baud38400.x + 6, cs_baud38400.y + 14);
      newbaud == SER_38400 ? putchar(18) : putchar(146);
      myputs(cs_baud38400.text);
      putchar(146);
    }

    redraw = 1;
    c = cgetc();

    switch(c) {

    case 13:
      *interface = newif;
      *baudrate = newbaud;
      closewindow(win);
      return(OK);
      break;

    case 3:
      closewindow(win);
      return(ERROR);
      break;

    case '1':
      newif = SER_TURBO232;
      break;

    case '2':
      newif = SER_SILVER;
      break;

    case '3':
      newbaud = SER_115200;
      break;

    case '4':
      newbaud = SER_57600;
      break;

    case '5':
      newbaud = SER_38400;
      break;

    default:
      redraw = 0;
      break;
    }
  }
}


/*

Draw petscii window

*/
void drawwindow(unsigned char *title, unsigned char x, unsigned char y, unsigned char w, unsigned char h, unsigned char flags, button *buttons[]) {
  unsigned char linebuf[85];
  unsigned char i;

  /* draw title */
  gotoxy(x, y);
  putchar(18);
  myputs(title);
  i = strlen(title);
  while (++i < w) {
    putchar(169);
  }
  if (flags & WIN_CLOSEBUTTON) {
    putchar(88);
  } else {
    putchar(169);
  }
  putchar(146);

  /* top border */
  gotoxy(x, y + 1);
  linebuf[0] = 18;
  linebuf[1] = 172;
  linebuf[w] = 187;
  for (i = 2; i < w; ++i) {
    linebuf[i] = 162;
  }
  linebuf[w+1] = 146;
  linebuf[w+2] = 0;
  myputs(linebuf);

  /* side borders */
  linebuf[0] = 161;
  for (i = 1; i < w - 1; ++i) {
    linebuf[i] = ' ';
  }
  linebuf[w-1] = 18;
  linebuf[w] = 161;
  linebuf[w+1] = 146;
  linebuf[w+2] = 0;
  for (i = 2; i < h - 1; ++i) {
    gotoxy(x, y + i);
    myputs(linebuf);
  }

  /* bottom border */
  gotoxy(x, y + h - 1);
  linebuf[0] = 18;
  linebuf[1] = 188;
  linebuf[2] = 146;
  linebuf[w+1] = 18;
  linebuf[w+2] = 190;
  linebuf[w+3] = 146;
  linebuf[w+4] = 0;
  for (i = 3; i < w + 1; ++i) {
    linebuf[i] = 162;
  }
  myputs(linebuf);

  /* buttons */
  if (buttons) {
    i = 1;
    while (*buttons != NULL) {
      gotoxy(x + w - i - strlen((*buttons)->text), y + h - 2);
      i += strlen((*buttons)->text) + 1;
      putchar(18);
      printf((*buttons)->text);
      putchar(146);
      ++buttons;
    }
  }
}


/*

return a screen pointer to x, y

*/
unsigned char *screenaddr(unsigned char x, unsigned char y) {
#ifdef __C64__
  unsigned char *screen = (unsigned char *) 0x0400;
  unsigned char w = 40;
#endif
#ifdef __C128__
  unsigned char *screen = (unsigned char *) 0x0400;
  unsigned char w = 40;
#endif
#ifdef __PLUS4__
  unsigned char *screen = (unsigned char *) 0x0c00;
  unsigned char w = 40;
#endif

  return(screen + y*w + x);
}


/*

Open a window, returns a handle

*/
window *openwindow(unsigned char *title, unsigned char x, unsigned char y, unsigned char w, unsigned char h, unsigned char flags, button *buttons[]) {
  window *win;
  unsigned char by;

  if ((win = malloc(sizeof(window))) == NULL) {
    return(NULL);
  }

  win->title = title;
  win->x = x;
  win->y = y;
  win->w = w;
  win->h = h;
  win->flags = flags;
  win->buttons = buttons;

  if ((win->backbuffer = malloc(w*h)) == NULL) {
    free(win);
    return(NULL);
  }

  for (by = 0; by < h; ++by) {
    memcpy(win->backbuffer + by*w, screenaddr(x, by + y), w);
  }

  drawwindow(title, x, y, w, h, flags, buttons);

  return(win);
}


/*

Close a window, free resources

*/
void closewindow(window *win) {
  unsigned char by;

  for (by = 0; by < win->h; ++by) {
    memcpy(screenaddr(win->x, by + win->y), win->backbuffer + by*win->w, win->w);
  }

  free(win->backbuffer);
  free(win);
}


/*

Print a character

*/
void __fastcall__ myputchar(unsigned char) {
  __asm__("jsr $ffd2");
}


/*

Print a string

*/
void myputs(unsigned char *string) {
  while(*string) {
    myputchar(*string++);
  }
}


button *req_ok[] = {
  &btnok,
  NULL
};

button *req_okcancel[] = {
  &btncancel,
  &btnok,
  NULL
};

button *req_yesno[] = {
  &btnno,
  &btnyes,
  NULL
};

/* Show a requester */
unsigned char showrequester(unsigned char *title, unsigned char *message, unsigned char buttons) {
  window *win;
  unsigned char i;
  button **b = NULL;
  unsigned char numbuts = 0;
  unsigned char xs, ys, w, c;

  if (buttons == REQ_OK) {
    b = req_ok;
    numbuts = 1;
  } else if (buttons == REQ_OKCANCEL) {
    b = req_okcancel;
    numbuts = 2;
  } else if (buttons == REQ_YESNO) {
    b = req_yesno;
    numbuts = 2;
  } else {
    return(0);
  }

  screensize(&xs, &ys);

  if (strlen(message) > 37) {
    message[37] = 0;
  }
  if (strlen(title) > 38) {
    title[38] = 0;
  }

  w = strlen(message) + 2;
  if (w < 15) {
    w = 15;
  }
  if (w < strlen(title) + 1) {
    w = strlen(title) + 1;
  }

  if ((win = openwindow(title, (xs - w) / 2, 9, w, 6, 0, b)) == NULL) {
    return(0);
  }

  gotoxy((xs - w) / 2 + 1, 11);
  myputs(message);

  while (1) {
    c = cgetc();

    for (i = 0; i < numbuts; ++i) {
      if (b[i]->key == c) {
	closewindow(win);
	return(numbuts - i);
      }
    }

  }
}


void printspaces(unsigned char count) {
  do {
    myputchar(' ');
  } while (--count > 0);
}


/*

Display directory list

*/
void showdir(directory *dir, unsigned char maxlines, unsigned char startline) {
  direntry *de;
  unsigned char shortname[17];
  unsigned char l = 0;

  shortname[15] = '*';
  shortname[16] = 0;
  gotoxy(0, startline);
  de = getentry(dir, dir->offset);
  while (l < maxlines) {
    if ((dir->offset) + l == (dir->current)) {
      myputchar(18);
    }
    if (strlen(de->name) > 16) {
      memcpy(shortname, de->name, 15);
      printf("\x1d\x1d%-5d \"%s\" ", de->size, shortname);
    } else {
      printf("\x1d\x1d%-5d \"%s\" ", de->size, de->name);
    }
    if (strlen(de->name) < 16) {
      printspaces(16 - strlen(de->name));
    }
    myputs(entrytype[de->type]);
    myputchar('\n');
    if (de->next) {
      de = de->next;
      ++l;
    } else {
      l = 255;
    }
  }
}


void printcurrent(directory *dir, unsigned char startline) {
  direntry *de;
  unsigned char shortname[17];

  gotoxy(0, startline + dir->current - dir->offset);
  de = getentry(dir, dir->current);
  if (strlen(de->name) > 16) {
    shortname[15] = '*';
    shortname[16] = 0;
    memcpy(shortname, de->name, 15);
    printf("\x1d\x1d%-5d \"%s\" ", de->size, shortname);
  } else {
    printf("\x1d\x1d%-5d \"%s\" ", de->size, de->name);
  }
  if (strlen(de->name) < 16) {
    printspaces(16 - strlen(de->name));
  }
  myputs(entrytype[de->type]);
  myputchar('\n');
}


void printrevcurrent(directory *dir, unsigned char startline) {
  myputchar(18);
  printcurrent(dir, startline);
}


void scrolldirup(unsigned char maxlines, unsigned char startline) {
  unsigned char l;

  for (l = startline; l < startline + maxlines; ++l) {
    memcpy(screenaddr(2, l), screenaddr(2, l + 1), 28);
  }
}


void scrolldirdown(unsigned char maxlines, unsigned char startline) {
  unsigned char l;

  for (l = startline + maxlines - 1; l > startline; --l) {
    memcpy(screenaddr(2, l), screenaddr(2, l - 1), 28);
  }
}


void cleardir(unsigned char maxlines, unsigned char startline) {
  unsigned char l;

  for (l = 0; l < maxlines; ++l) {
    gotoxy(2, l + startline);
    myputs("                            ");
  }
}
