;---------------------------------------
; Serial Slave driver starter
;
; Sorta userfriendly way of loading the
; Serial Slave driver into memory
;
; History
;   0.1  C64 version implemented


	processor 6502


	include "kernal.i"
	include "ssconst.i"


;---------------------------------------
; BASIC stub

 ifconst C64
	org $0801

	dc.b $0b,$08
	dc.w 2002
	dc.b $9e	; SYS
	dc.b "2061"
	dc.b 0	; EOL
	dc.w 0	; EOF

drvaddr	equ $ce00
zp	equ $fb

 else
 ifconst C128
	org $1c01

	dc.b $0b,$1c
	dc.w 2002
	dc.b $9e	; SYS
	dc.b "7181"
	dc.b 0	; EOL
	dc.w 0	; EOF

drvaddr	equ $1a00
zp	equ $fb

 else
 ifconst PLUS4
	org $1001

	dc.b $0b,$10
	dc.w 2002
	dc.b $9e	; SYS
	dc.b "4109"
	dc.b 0	; EOL
	dc.w 0	; EOF

drvaddr	equ $xxxx
zp	equ $xx

 else
 ifconst PET
	org $0401

	dc.b $0b,$04
	dc.w 2002
	dc.b $9e	; SYS
	dc.b "1037"
	dc.b 0	; EOL
	dc.w 0	; EOF

drvaddr	equ $xxxx
zp	equ $xx

 endif
 endif
 endif
 endif


;---------------------------------------
; INIT

	jsr primm	; banner
	dc.b 13
	dc.b "SERIAL SLAVE DRIVER V0.5",13
	dc.b 13
	dc.b "SELECT SERIAL INTERFACE:",13
	dc.b "  1) SWIFTLINK",13
	dc.b "  2) TURBO232",13
 ifconst C64
	dc.b "  3) SILVER SURFER",13
 endif
	dc.b "  0) QUIT",13
	dc.b 0


waitnum	jsr getnum	; wait for a number
	bmi waitnum

	cmp #0	; parse
	beq quit
	cmp #1
	beq swiftlink
	cmp #2
	beq turbo232
 ifconst C64
	cmp #3
	beq silversurfer
 endif
	jmp waitnum


quit	rts


swiftlink	jsr primm
	dc.b 13,"SWIFTLINK",0
	lda #<(sl+2)	; +2 to skip the load address
	sta zp
	lda #>(sl+2)
	sta zp+1
	jmp copydriver

turbo232	jsr primm
	dc.b 13,"TURBO232",0
	lda #<(t232+2)
	sta zp
	lda #>(t232+2)
	sta zp+1
	jmp copydriver

 ifconst C64
silversurfer	jsr primm
	dc.b 13,"SILVER SURFER",0
	lda #<(silver+2)
	sta zp
	lda #>(silver+2)
	sta zp+1
	jmp copydriver
 endif


copydriver	ldy #0	; copy the 512 byte driver
copy1	lda (zp),y
	sta drvaddr,y
	iny
	bne copy1
	inc zp+1
copy2	lda (zp),y
	sta drvaddr+$100,y
	iny
	bne copy2

	jsr primm	; print status message
	dc.b " DRIVER INSTALLED ON DEVICE ",0

	ldx drvaddr+3	; print device number
	lda #0
	jsr print16

	jsr primm
	dc.b 13,"SYS ",0

	ldx #<drvaddr	; print sys address
	lda #>drvaddr
	jsr print16

	jsr primm
	dc.b " TO RESTART",13,0

	jmp drvaddr	; load driver and quit


sl	incbin "ss64-slink.prg"
t232	incbin "ss64-t232.prg"
 ifconst C64
silver	incbin "ss64-silver.prg"
 endif


; wait for 0-9 on the keyboard
getnum	jsr getin
	cmp #47
	bcc getnum
	cmp #57
	bcs getnum
	and #$0f
	rts


;---------------------------------------
; print16 and primm
	include "util.i"
