use strict;


# read and parse the config file
sub readconfig($) {
  my $cfgfile = shift;
  my $ln = 0;
  my ($l, $keyword, $value);

  unless (-f $cfgfile) {
    print "no config file, using defaults\n";
    return(0);
  }

  unless (open(CFG, $cfgfile)) {
    print "couldn't open $cfgfile, using defaults\n";
    return(0);
  }

  while ($l = <CFG>) {
    $ln++;                   # line number
    $l =~ s/[\r\n]+$//;      # purge crlf
    $l =~ s/\s*#.*$//;       # purge comments
    next if ($l =~ /^\s*$/); # empty?

    # should match keyword = value
    unless ($l =~ /^\s*([a-z0-9_]+)\s*=\s*(.+)\s*$/i) {
      print "error in $cfgfile line $ln\n";
      goto RC_DONE;
    }
    handlekwvl(lc($1), $2, $cfgfile, $ln, 1);

  }

 RC_DONE:
  close(CFG);
}


# handle a keyword and value pair
sub handlekwvl($$$$) {
  my $keyword = shift;
  my $value = shift;
  my $cfgfile = shift;
  my $ln = shift;
  my $warn = shift;

  # port = /dev/ttyS0
  if ($keyword eq "port") {
    if ($warn && defined($::port)) {
      print "redefine in $cfgfile line $ln\n";
    }
    $::port = $value;
  }

  # baudrate = 115200
  elsif ($keyword eq "baudrate") {
    if ($warn && defined($::baudrate)) {
      print "redefine in $cfgfile line $ln\n";
    }
    unless ($value =~ /^\d+$/) {
      print "illegal value " . pretty($value) . " in $cfgfile line $ln\n";
    }
    $::baudrate = $value;
  }

  # something else
  else {
    print "unknown keyword in $cfgfile line $ln: $keyword\n";
  }
}


return(1);
